<meta http-equiv="content-type" content="application/xhtml+xml; charset=utf-8" />

Disseny i implementació d'una base de dades relacional per a un concessionari de vehicles
=========================================================================================

Introducció al directori code
-----------------------------

Aquest directori conté el codi escrit per l'aplicació. En aquest fitxer es descriu la feina feta per cada tasca amb els detalls i explicacions pertinents a cada decisió.
En general, s'ha de tenir present el següent:

- **Codificació dels noms d'objectes**: Donat que la implementació es fa en ASCII, molts dels noms que s'han utilitzat per a representar relacions, funcions, procediments, etcètera, s'han adaptat.
- **Nomenclatures**: En quant a la forma de les nomenclatures, s'ha seguit el mètode [Camel Case][Camel Case]. Pel que fa al seu contingut, s'ha procurat que siguin el més senzilles, menys redundants i més clares possible. Així, si un camp és el codi de l'entitat casa, s'anomena codi, peró si un camp és una clau forània que fa referència a l'entitat ciutat, s'anomenarà codiCiutat.

Tasca 50: Implementar els tests per la compra d'un vehicle
----------------------------------------------------------

El primer pas d'aquesta tasca ha estat crear el model físic al servidor. El model s'ha implementat tenint en compte els següents detalls:

- S'ha creat un _tablespace_ per la base de dades anomenat `concessionari`. Cal indicar que s'ha configurat aquest _tablespace_ per permetre'n el creixement automàticament.
- S'ha creat l'esquema `concessionari`, amb la contrasenya `concessionari` i se li otorguen els permissos necessaris per poder-se connectar i administrar el _tablespace_.
- Connectant-se al nou esquema, es creen les taules.
- Per implementar la clau primària amb increment automàtic de la taula `botigues`, es crea una seqüència que va generant els identificadors i un _trigger_ que actúa en els inserts sobre la taula, donant-li el valor al camp numèric `codi`.
- El camp `codiBotiga` de la taula `empleats` és una clau forània que fa referència al camp `codi` de la taula `botigues`.
- La taula compres té les següents peculiaritats:
> - El camp `nifEmpleat` és una clau forània que referencia al camp `nif` de la taula `empleats`.
> - El camp `nifVenedor` és una clau forània referent al camp `nif` de la taula `venedors`.
> - El camp `vinVehicle` és una clau forània que fa referència al camp `vin` de la taula `vehicles`.
> - La clau primària, que s'anomena `pk_compres` és composta pels camps `nifEmpleat`, `nifVenedor`, `vinVehicle` i `data`.

Per tal de facilitar-ne la gestió del model, s'han preparat uns guions SQL que permeten crear, esborrar i reiniciar el model. Es troben al directori code, s'anomenen `create.sql`, `delete.sql`, `reset.sql` i es poden invocar des de l'SQL\*Plus de la forma següent:

    SQL> @/camí/al/repositori/code/create.sql

Tot seguit, per la implementació dels scripts de test, s'ha fet servir la llibreria PLUTO per implementar els tests, utilitzant els procediments descrits a la tasca 49. El codi resultant, ha quedat escrit en un fitxer dins del directori de codi, que es pot carregar al SQL\*Plus amb la següent instrucció:

    SQL> @/camí/al/repositori/code/tests/tasca50.sql

Finalment, s'ha implementat una altra recepta més de chef, que instal·la el mercurial, clona el repositori local del hoste i instal·la el model i els tests. Evidentment, per a que aquesta recepta funcioni, caldrà que, abans d'aixecar el sistema hospedat, s'arrenqui el servidor de mercurial, executant les comandes següentse, des del directori del repositori:

    $ hg serve &
    $ cd test
    $ vagrant up

Tasca 60: Implementar els tests per la venda d'un vehicle
----------------------------------------------------------

De forma similar a la tasca 50, en la què es crearen els tests per la compra de vehicles, en aquesta s'han creat les estructures necessàries per a poder implementar els tests. Aquestes estructures són:

- Les taules necessàries per les vendes.
- Els procediments especificats, però que no realitzen res.

Amb això fet, s'han implementat els tests dels dos procediments especificats. Finalment, s'han integrat amb els guions que ja hi havia preparats de la iteració anterior.

Tasca 53: Implementar tests per validar el registre de les accions de l'aplicació
---------------------------------------------------------------------------------

Com en la tasca d'implementar els tests de les compres, primer s'ha creat el model. De fet, s'ha ampliat el codi que hi ha a `create.sql` amb el codi necessari per crear les taules `registres` i `atributs`, creant una nova seqüència i disparador pel camp `codi` de `registres`. La taula `atributs` s'ha creat tenint present la clau forània `codiRegistre` i la clau primària composta per `codiRegistre` i `nom`.

Tot seguit, s'ha vist que la funció log, en realitat, ja existeix, així que s'ha substituït per registra, amb el mateix prototipus i s'ha implementat el test tenint en compte aquest detall.

Val a dir que per a la utilització del segon paràmetre, cal crear una taula PL/SQL, com es descriu a [Tablas PL/SQL][Tablas PL/SQL], on es depositen tots els atributs.

Finalment, s'han modificat les receptes per tal que incloguin la creació dels objectes de test pertanyents a aquesta tasca.

Tasca 51: Implementar la compra de vehicles
-------------------------------------------

En aquesta tasca s'han implementat els procediments compra i nouVenedor, que satisfan els tests implementats prèviament.

Tasca 61: Implementar la venda d'un vehicle
-------------------------------------------

En aquesta tasca s'han implementat els procediments venda i nouClient, satisfent els tests implementats prèviament.

Tasca 55: Implementar el registre d'accions de l'aplicació
----------------------------------------------------------

En primer lloc, per completar la tasca, s'ha creat i implementat el procediment `registra`, per això, s'han creat dos tipus nous dins de l'esquema CONCESSIONARI: `atribut` i `tAtributs`, que són, respectivament, un objecte i un vector d'`atribut`s que permeten gestionar els paràmetres del procediment. En variar el prototip del procediment, ha calgut modificar el test implementat en la tasca 53, peró, un cop corregit, s'ha pogut validar la implementació realitzada.
Finalment, també s'han modificat els procediments `compra` i `nouVenedor` per què en facin ús del registre d'accions.

Tasca 56: Implementar les validacions de les estadístiques de la compra de vehicles
-----------------------------------------------------------------------------------

Per tal d'implementar les validacions en condicions, en primer lloc s'han afegit, al guió `create.sql`, les estructures detallades al disseny de les estructures per les estadístiques. Aquestes estructures reuneixen les següents característiques:

* El model pel magatzem de dades s'ha dissenyat en un esquema separat.
* Totes les dimensions i la taula de fets tenen un identificador numèric autoincremental separat dels que s'utilitzen en l'aplicació.
* Com no s'han definit consultes concretes per les compres, s'ha considerat que es comptaran les compres únicament, peró tampoc s'han considerat taules agregades. En tractar les posteriors històries d'usuari, es podrien detectar més canvis addicionals que comportin noves estructures.

Finalment, s'ha implementat el test bàsic plantejar per les compres.

Tasca 63: Implementar les validacions de les estadístiques de la venda de vehicles
-----------------------------------------------------------------------------------

En aquesta tasca s'ha implementat un codi de test que valida conjuntament les estadístiques de les compres i de les vendes, per tant, aquest nou codi **substitueix** el de la tasca 56, que només validava les estadístiques de les compres. D'altra banda, ha fet falta implementar les dimensions i la taula de fets previstes. A més, s'ha modificat el codi del procediment `etl` de forma que en cas de trobar-se amb funcions no registrades, no dòna errors.

Tasca 57: Implementar les estadístiques de la compra de vehicles
----------------------------------------------------------------

Bàsicament, en aquesta tasca s'ha implementat el procés principal _ETL_ i la part concreta que afecta a la càrrega de les dades de les compres. L'aproximació triada és una mica allunyada de l'habitual, ja que es disposa d'un registre de totes les accions preses.

* **Per l'_ETL_**: El procés principal d'actualització del magatzem de dades s'encarrega de trobar la informació que cal carregar a partir del registre d'accions. El primer que fa és determinar quan és l'últim cop que el procés es va executar, cercant, al registre, la seva entrada més recent. Si no en troba, assumeix que ha de carregar totes les dades que hi ha al registre. Si en troba, només sel·leccionarà les entrades que hi ha a partir d'aquesta última execució.
  Per cada entrada que ha de carregar, mira el nom de la funció i segons el nom que té, executarà un o altre procediment passant-li el codi de registre per tal que pugui recuperar els atributs que necessiti.
* **Per les compres**: El procediment _ETL_ de compres, s'encarrega de recollir els atributs vinculats al registre que s'està carregant i, si cal integrar-los al magatzem. Per fer això, es comporta d'una forma similar al procés general d'_ETL_, és a dir, segons l'atribut que s'està carregant, executa uns procediments que netegen i adapten la informació, actualitzen la dimensió corresponent, si no existia; i, retornen el registre que hi ha a la dimensió corresponent. El procediment d'_ETL_ de compres, en diferència amb el general, va mantenint un registre de la taula de fets de compres en memòria, que va completant a mida que va tractant els valors d'atributs per, en acabat, insertar-lo a la taula de fets corresponent.

Tasca 64: Implementar les estadístiques de la venda de vehicles
---------------------------------------------------------------

Continuant amb el disseny de procés començat en la tasca 57, s'han afegit els procediments per les vendes i pel tractament dels nous client, també.
En diferència a la tasca 57, també s'han implementat les vistes per les consultes demanades a la història d'usuari.

[Camel Case]: http://es.wikipedia.org/wiki/CamelCase "Definició de Camel Case, en castellà, última visita 29/10/2012"
[Tablas PL/SQL]: http://www.devjoker.com/contenidos/Tutorial-PLSQL/59/Tablas-PLSQL.aspx "Explicació sobre les taules PL/SQL, en castellà, última visita 01/11/2012"
