SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('Creant procediments ETL');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNullVarchar2');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Funci per transformar una cadena que val nul.
CREATE OR REPLACE FUNCTION etlNullVarchar2 (valor IN VARCHAR2) RETURN VARCHAR2 IS
BEGIN
  -- Si el valor s null,
  IF valor IS NULL THEN
    -- retorna una cadena buida;
    RETURN ' ';
  -- altrament,
  ELSE
    -- retorna el valor.
    RETURN valor;
  END IF;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNouEmpleat');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i registrar un empleat al magatzem.
CREATE OR REPLACE FUNCTION etlNouEmpleat (nifEmpleat IN VARCHAR2) RETURN dimEmpleats%rowtype IS
  empleat concessionari.empleats%rowtype;
  dimEmpleat concessionariDWH.dimEmpleats%rowtype;
BEGIN
  -- Recupera les dades de la base de dades OLTP.
  SELECT *
    INTO empleat
    FROM concessionari.empleats
    WHERE nif = nifEmpleat;
  -- Recupera les dades que hi pugui haver al magatzem.
  SELECT *
    INTO dimEmpleat
    FROM concessionariDWH.dimEmpleats
    WHERE nif = empleat.nif
      AND nom = empleat.nom;
  -- Retorna l'empleat trobat al magatzem
  RETURN(dimEmpleat);
EXCEPTION
  -- Si no hi ha cap empleat al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- el crea,
    INSERT INTO concessionariDWH.dimEmpleats VALUES (NULL, empleat.nif, empleat.nom, NULL);
    -- i el recupera, per
    SELECT *
      INTO dimEmpleat
      FROM concessionariDWH.dimEmpleats
      WHERE nif = empleat.nif
        AND nom = empleat.nom;
    -- retornar-lo.
    RETURN(dimEmpleat);
END etlNouEmpleat;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlBotigaEmpleat');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar la botiga a la que pertany un empleat, al magatzem.
CREATE OR REPLACE FUNCTION etlBotigaEmpleat(nifEmpleat IN VARCHAR2) RETURN dimBotigues%rowtype IS
  botiga concessionari.botigues%rowtype;
  dimBotiga concessionariDWH.dimBotigues%rowtype;
BEGIN
  -- Recupera la botiga de la base de dades OLTP.
  SELECT *
    INTO botiga
    FROM concessionari.botigues
    WHERE codi = (
                  SELECT codiBotiga
                  FROM concessionari.empleats
                  WHERE nif = nifEmpleat);
  -- Recupera les dades d'una botiga que hi hagi al magatzem.
  SELECT *
    INTO dimBotiga
    FROM concessionariDWH.dimBotigues
    WHERE codi = botiga.codi
      AND nom = botiga.nom
      AND adreca = botiga.adreca
      AND telefon = botiga.telefon
      AND fax = botiga.fax
      AND correue = botiga.correue;
  -- Retorna la botiga trobada.
  RETURN(dimBotiga);
EXCEPTION
  -- Si no es troba la botiga al magatzem, 
  WHEN NO_DATA_FOUND THEN
    -- la crea,
    INSERT INTO concessionariDWH.dimBotigues(codi,nom,adreca,telefon,fax,correue)
      VALUES (botiga.codi,botiga.nom,botiga.adreca,botiga.telefon,botiga.fax,botiga.correue);
    -- la recupera,
    SELECT *
      INTO dimBotiga
      FROM concessionariDWH.dimBotigues
      WHERE codi = botiga.codi
        AND nom = botiga.nom
	AND adreca = botiga.adreca
	AND telefon = botiga.telefon
	AND fax = botiga.fax
	AND correue = botiga.correue;
    -- i la retorna.
    RETURN(dimBotiga);
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNouVenedor');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar un nou venedor, al magatzem.
CREATE OR REPLACE FUNCTION etlNouVenedor (nifVenedor IN VARCHAR2) RETURN dimVenedors%rowtype IS
  venedor concessionari.venedors%rowtype;
  dimVenedor concessionariDWH.dimVenedors%rowtype;
BEGIN
  -- Recupera el venedor del sistema OLTP.
  SELECT *
    INTO venedor
    FROM concessionari.venedors
    WHERE nif = nifVenedor;
  -- Recupera el venedor del magatzem, si hi s.
  SELECT *
    INTO dimVenedor
    FROM concessionariDWH.dimVenedors
    WHERE nif = venedor.nif
      AND nom = venedor.nom;
  -- Retorna el venedor recuperat.
  RETURN(dimVenedor);
EXCEPTION
  -- Si el venedor no existeix al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- el crea,
    INSERT INTO concessionariDWH.dimVenedors
      VALUES (NULL,venedor.nif,venedor.nom,venedor.adreca,venedor.telefon,venedor.fax,venedor.correue);
    -- el recupera,
    SELECT *
      INTO dimVenedor
      FROM concessionariDWH.dimVenedors
      WHERE nif = venedor.nif
        AND nom = venedor.nom;
    -- i el retorna.
    RETURN(dimVenedor);
END etlNouVenedor;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNouVehicle');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar un nou vehicle, al magatzem.
CREATE OR REPLACE FUNCTION etlNouVehicle (vinVehicle2 IN VARCHAR2) RETURN dimVehicles%rowtype IS
  vehicle concessionari.vehicles%rowtype;
  matricula VARCHAR2(7);
  dimVehicle concessionariDWH.dimVehicles%rowtype;
BEGIN
  -- Recupera el vehicle del sistema OLTP.
  SELECT *
  INTO vehicle
  FROM concessionari.vehicles
  WHERE vin = vinVehicle2;
  SELECT matricula
  INTO matricula
  FROM concessionari.matricules
  WHERE vinVehicle = vinVehicle2;
  -- Recupera el vehicle del magatzem, si hi s.
  SELECT *
    INTO dimVehicle
    FROM concessionariDWH.dimVehicles
    WHERE vin = vehicle.vin
      AND matricula = matricula
      AND marca = vehicle.marca
      AND model = vehicle.model
      AND motor = vehicle.motor
      AND equipament = vehicle.equipament
      AND color = vehicle.color;
  -- Retorna el vehicle recuperat.
  RETURN(dimVehicle);
EXCEPTION
  -- Si el vehicle no existeix al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- el crea,
    INSERT INTO concessionariDWH.dimVehicles
      VALUES (
        NULL,
        vehicle.vin,
        matricula,
        vehicle.marca,
        vehicle.model,
        vehicle.motor,
        vehicle.equipament,
        vehicle.color,
        vehicle.kms
    );
    -- el recupera,
    SELECT *
      INTO dimVehicle
      FROM concessionariDWH.dimVehicles
      WHERE vin = vehicle.vin
      AND marca = vehicle.marca
      AND model = vehicle.model
      AND motor = vehicle.motor
      AND equipament = vehicle.equipament
      AND color = vehicle.color
      AND kms = vehicle.kms;
    -- i el retorna.
    RETURN(dimVehicle);
END etlNouVehicle;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNovaData');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una nove data, al magatzem.
CREATE OR REPLACE FUNCTION etlNovaData (dataNova IN VARCHAR2) RETURN dimTemps%rowtype IS
  dimData concessionariDWH.dimTemps%rowtype;
  dimData2 concessionariDWH.dimTemps%rowtype;
BEGIN
  -- Preparar les transformacions necessries per la data.
  dimData2.id := NULL;
  -- Transforma el valor a tipus data.
  dimData2.dataDia := TO_DATE(dataNova,'DD-MON-YY');
  -- Extreu l'any de la data.
  dimData2.anyData := TO_NUMBER(TO_CHAR(dimData2.dataDia, 'YYYY'),'9999');
  -- Extreu el trimestre de la data.
  dimData2.trimestre := TO_NUMBER(TO_CHAR(dimData2.dataDia, 'Q'),'9');
  -- Extreu el mes.
  dimData2.mes := TO_CHAR(dimData2.dataDia, 'MONTH');
  -- Extreu la setmana.
  dimData2.setmana := TO_NUMBER(TO_CHAR(dimData2.dataDia, 'WW'),'99');
  -- Extreu el dia de la setmana.
  dimData2.diaSetmana := TO_CHAR(dimData2.dataDia, 'DAY');
  -- Recupera la data del magatzem si existeix.
  SELECT *
    INTO dimData
    FROM concessionariDWH.dimTemps
    WHERE dataDia = dimData2.dataDia;
  -- Retorna la data.
  RETURN(dimData);
EXCEPTION
  -- Si la data no existeix al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- el crea.
    INSERT INTO concessionariDWH.dimTemps
      VALUES (
        NULL,
        dimData2.dataDia,
        dimData2.anyData,
        dimData2.trimestre,
        dimData2.mes,
        dimData2.setmana,
        dimData2.diaSetmana
    );
    -- el recupera.
    SELECT *
      INTO dimData
      FROM concessionariDWH.dimTemps
      WHERE dataDia = dimData2.dataDia;
    -- i el retorna.
    RETURN(dimData);
END etlNovaData;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlCompra');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una compra, al magatzem.
CREATE OR REPLACE PROCEDURE etlCompra(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
  botiga concessionariDWH.dimBotigues%rowtype;
  venedor concessionariDWH.dimVenedors%rowtype;
  vehicle concessionariDWH.dimVehicles%rowtype;
  dataCompra concessionariDWH.dimTemps%rowtype;
  fetCompra concessionariDWH.fetsCompres%rowtype;
BEGIN
  -- Per cada atribut relacionat amb el registre de la compra,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF de l'empleat,
      WHEN 'nifEmpleat' THEN
        -- obtenir el registre de la dimensi l'empleat.
        empleat := etlNouEmpleat(atribut.valor);
        -- Obtenir el registre de la botiga corresponent.
        botiga := etlBotigaEmpleat(atribut.valor);
        -- Guardar els identificadors, tant de l'empleat com de la botiga.
	fetCompra.idEmpleat := empleat.id;
	fetCompra.idBotiga := botiga.id;
      -- Si s el NIF del venedor,
      WHEN 'nifVenedor' THEN
        -- obtenir el registre de la dimensi del venedor.
        venedor := etlNouVenedor(atribut.valor);
        -- Guardar l'identificador del venedor.
	fetCompra.idVenedor := venedor.id;
      -- Si s el VIN del vehicle,
      WHEN 'vin' THEN
        -- obtenir el registre de la dimensi del vehicle.
        vehicle := etlNouVehicle(atribut.valor);
        -- Guardar l'identificador del vehicle.
        fetCompra.idVehicle := vehicle.id;
      -- Si s la data de compra,
      WHEN 'data' THEN
        -- obtenir el registre de la dimensi de dates.
        dataCompra := etlNovaData(atribut.valor);
        -- Guardar l'identificador del vehicle.
	fetCompra.idTemps := dataCompra.id;
      -- Si es tracta de l'import,
      WHEN 'import' THEN
        -- Es fa la transformaci a nombre i es guarda.
        fetCompra.import := TO_NUMBER(atribut.valor,'99999999.99');
      -- Si el nom de l'atribut s desconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- S'inserten les dades de la compra a la taula de fets corresponents.
  INSERT INTO concessionariDWH.fetsCompres
    VALUES (
      NULL,
      fetCompra.idBotiga,
      fetCompra.idEmpleat,
      fetCompra.idVenedor,
      fetCompra.idVehicle,
      fetCompra.idTemps,
      fetCompra.import
  );
END etlCompra;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNouComercial');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar un nou comercial, al magatzem.
CREATE OR REPLACE FUNCTION etlNouComercial(nifComercial IN VARCHAR2) RETURN dimComercials%rowtype IS
  comercial concessionari.comercials%rowtype;
  dimComercial concessionariDWH.dimComercials%rowtype;
BEGIN
  -- Recupera el comercial del sistema OLTP.
  SELECT *
    INTO comercial
    FROM concessionari.comercials
    WHERE nifEmpleat = nifComercial;
  -- Recupera el comercial del magatzem, si hi s.
  SELECT *
    INTO dimComercial
    FROM concessionariDWH.dimComercials
    WHERE nif = comercial.nifEmpleat;
  -- Retorna el comercial recuperat.
  RETURN(dimComercial);
EXCEPTION
  -- Si el comercial no existeix al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- el crea,
    INSERT INTO concessionariDWH.dimComercials VALUES (NULL,comercial.nifEmpleat);
    -- el recupera,
    SELECT *
      INTO dimComercial
      FROM concessionariDWH.dimComercials
      WHERE nif = comercial.nifEmpleat;
    -- i el retorna.
    RETURN(dimComercial);
END etlNouComercial;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlClient');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar un nou client, al magatzem.
CREATE OR REPLACE FUNCTION etlClient(nifClient IN VARCHAR2) RETURN dimClients%rowtype IS
  client concessionari.clients%rowtype;
  dimClient concessionariDWH.dimClients%rowtype;
BEGIN
  -- Recupera el client del sistema OLTP.
  SELECT *
    INTO client
    FROM concessionari.clients
    WHERE nif = nifClient;
  -- Recupera el client del magatzem, si hi s.
  SELECT *
    INTO dimClient
    FROM concessionariDWH.dimClients
    WHERE nif = client.nif
    AND nom = client.nom;
  -- Retorna el client recuperat.
  RETURN(dimClient);
EXCEPTION
  -- Si el client no existeix al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- el crea,
    INSERT INTO concessionariDWH.dimClients
      VALUES (NULL,client.nif,client.nom,client.adreca,client.telefon,client.fax,client.correue);
    -- el recupera,
    SELECT *
      INTO dimClient
      FROM concessionariDWH.dimClients
      WHERE nif = client.nif
        AND nom = client.nom;
    -- i el retorna.
    RETURN(dimClient);
END etlClient;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlVenda');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una nova venda, al magatzem.
CREATE OR REPLACE PROCEDURE etlVenda(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
  comercial concessionariDWH.dimComercials%rowtype;
  botiga concessionariDWH.dimBotigues%rowtype;
  client concessionariDWH.dimClients%rowtype;
  vehicle concessionariDWH.dimVehicles%rowtype;
  dataVenda concessionariDWH.dimTemps%rowtype;
  fetVenda concessionariDWH.fetsVendes%rowtype;
BEGIN
  -- Per cada atribut relacionat amb la venda,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF del comercial,
      WHEN 'nifComercial' THEN
        -- obtenir el registre de la dimensi de l'empleat.
        empleat := etlNouEmpleat(atribut.valor);
        -- Obtenir el registre de la dimensi de comercials.
	comercial := etlNouComercial(atribut.valor);
        -- Obtenir el registre de la dimensi la botiga.
	botiga := etlBotigaEmpleat(atribut.valor);
        -- Guardar l'identificador del comercial i de la botiga.
	fetVenda.idComercial := comercial.id;
	fetVenda.idBotiga := botiga.id;
      -- Si s el NIF del client,
      WHEN 'nifClient' THEN
        -- obtenir el registre de la dimensi del client.
        client := etlClient(atribut.valor);
        -- Guardar l'identificador del client.
	fetVenda.idClient := client.id;
      -- Si s el VIN,
      WHEN 'vin' THEN
        vehicle := etlNouVehicle(atribut.valor);
	fetVenda.idVehicle := vehicle.id;
      -- Si s la data de venda,
      WHEN 'data' THEN
        dataVenda := etlNovaData(atribut.valor);
	fetVenda.idTemps := dataVenda.id;
      -- Si s l'import,
      WHEN 'import' THEN
        fetVenda.import := TO_NUMBER(atribut.valor,'99999999.99');
      -- Si qualsevol altre atribut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- S'inserten les dades de la compra a la taula de fets corresponents.
  INSERT INTO concessionariDWH.fetsVendes
    VALUES (
      NULL,
      fetVenda.idBotiga,
      fetVenda.idComercial,
      fetVenda.idClient,
      fetVenda.idVehicle,
      fetVenda.idTemps,
      fetVenda.import
  );
END etlVenda;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNouClient');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar un nou client, al magatzem.
CREATE OR REPLACE PROCEDURE etlNouClient(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  client concessionariDWH.dimClients%rowtype;
  client2 concessionariDWH.dimClients%rowtype;
BEGIN
  -- Per cada atribut relacionat amb el nou client,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF,
      WHEN 'nif' THEN
        -- es guarda el NIF.
        client.nif := atribut.valor;
      -- Si s el nom,
      WHEN 'nom' THEN
        -- es guarda el nom.
        client.nom := atribut.valor;
      -- Si s l'adrea,
      WHEN 'adreca' THEN
        -- es guarda l'adrea.
        client.adreca := atribut.valor;
      -- Si s el telfon,
      WHEN 'telefon' THEN
        -- es guarda el telfon.
        client.telefon := atribut.valor;
      -- Si s el fax,
      WHEN 'fax' THEN
        -- es guarda el fax.
        client.fax := atribut.valor;
      -- Si s el correu,
      WHEN 'correuE' THEN
        -- es guarda el correu.
        client.correue := atribut.valor;
      -- Si no s cap atribut conegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- Recuperar el client del magatzem, si hi s.
  SELECT *
    INTO client2
    FROM concessionariDWH.dimClients
    WHERE nif = client.nif
      AND nom = client.nom
      AND adreca = client.adreca
      AND telefon = client.telefon
      AND fax = client.fax
      AND correue = client.correue;
EXCEPTION
  -- Si no hi ha el client,
  WHEN NO_DATA_FOUND THEN
    -- el crea.
    INSERT INTO concessionariDWH.dimClients
      VALUES (
        NULL,
        client.nif,
        client.nom,
        client.adreca,
        client.telefon,
        client.fax,
        client.correue
    );
END etlNouClient;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNouMecanic');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar un nou mecnic, al magatzem.
CREATE OR REPLACE FUNCTION etlNouMecanic(nifMecanic IN VARCHAR2) RETURN dimMecanics%rowtype IS
  mecanic concessionari.mecanics%rowtype;
  dimMecanic concessionariDWH.dimMecanics%rowtype;
BEGIN
  -- Recupera el mecnic del sistema OLTP.
  SELECT *
    INTO mecanic
    FROM concessionari.mecanics
    WHERE nifEmpleat = nifMecanic;
  -- Recupera el mecnic del magatzem.
  SELECT *
    INTO dimMecanic
    FROM concessionariDWH.dimMecanics
    WHERE nif = mecanic.nifEmpleat;
  -- Retorna el mecnic.
  RETURN(dimMecanic);
EXCEPTION
  -- Si no hi s el mecnic al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- el crea,
    INSERT INTO concessionariDWH.dimMecanics VALUES (NULL,mecanic.nifEmpleat);
    -- el recupera,
    SELECT *
      INTO dimMecanic
      FROM concessionariDWH.dimMecanics
      WHERE nif = mecanic.nifEmpleat;
    -- i el retorna.
    RETURN(dimMecanic);
END etlNouMecanic;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlNovaHora');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una nova hora, al magatzem.
CREATE OR REPLACE FUNCTION etlNovaHora (horaNova IN VARCHAR2) RETURN dimHores%rowtype IS
  dimTemps concessionariDWH.dimTemps%rowtype;
  dimHora concessionariDWH.dimHores%rowtype;
  dimHora2 concessionariDWH.dimHores%rowtype;
BEGIN
  -- Prepara les transformacions a la nova hora.
  dimHora2.id := NULL;
  -- Transforma el valor a la marca de temps.
  dimHora2.temps := to_timestamp(horaNova,'DD-MON-YY HH.MI.SS.FF9 AM');
  -- Obt una referncia a la dimensi de dates, per la part de data.
  dimTemps := etlNovaData(to_char(dimHora2.temps,'DD-MON-YY'));
  dimHora2.idTemps := dimTemps.id;
  -- Transforma per aconseguir l'hora.
  dimHora2.hora := TO_NUMBER(TO_CHAR(dimHora2.temps,'HH'));
  -- Transforma per aconseguir el minut.
  dimHora2.minut := TO_NUMBER(TO_CHAR(dimHora2.temps,'MI'));
  -- Transforma per aconseguir el segon.
  dimHora2.segon := TO_NUMBER(TO_CHAR(dimHora2.temps,'SS'));
  -- Recupera l'hora del magatzem.
  SELECT *
    INTO dimHora
    FROM concessionariDWH.dimHores
    WHERE temps = dimHora2.temps;
  -- Retorna l'hora recuperada.
  RETURN(dimHora);
EXCEPTION
  -- Si no hi ha l'hora al magatzem,
  WHEN NO_DATA_FOUND THEN
    -- la crea,
    INSERT INTO concessionariDWH.dimHores
      VALUES (
        NULL,
        dimHora2.idTemps,
        dimHora2.temps,
        dimHora2.hora,
        dimHora2.minut,
        dimHora2.segon
    );
    -- la recupera,
    SELECT *
      INTO dimHora
      FROM concessionariDWH.dimHores
      WHERE temps = dimHora2.temps;
    -- i la retorna.
    RETURN(dimHora);
END etlNovaHora;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlEntraRevisio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una entrada a revisi, al magatzem.
CREATE OR REPLACE PROCEDURE etlEntraRevisio(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
  mecanic concessionariDWH.dimMecanics%rowtype;
  botiga concessionariDWH.dimBotigues%rowtype;
  client concessionariDWH.dimClients%rowtype;
  vehicle concessionariDWH.dimVehicles%rowtype;
  horaEntrada concessionariDWH.dimHores%rowtype;
  revisio concessionariDWH.fetsRevisions%rowtype;
BEGIN
  -- Per cada atribut relacionat amb l'entrada,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF del mecnic,
      WHEN 'nifMecanic' THEN
        -- recupera l'empleat,
        empleat := etlNouEmpleat(atribut.valor);
        -- el mecnic,
	mecanic := etlNouMecanic(atribut.valor);
        -- i la botiga.
	botiga := etlBotigaEmpleat(atribut.valor);
        -- Guarda el mecnic i la botiga.
	revisio.idMecanic := mecanic.id;
	revisio.idBotiga := botiga.id;
      -- Si s el NIF del client,
      WHEN 'nifClient' THEN
        -- recupera el client.
        client := etlClient(atribut.valor);
        -- Guarda la referncia del client.
	revisio.idClient := client.id;
      -- Si s el VIN del vehicle,
      WHEN 'vin' THEN
        -- Recupera el vehicle.
        vehicle := etlNouVehicle(atribut.valor);
        -- Guarda la referncia del vehicle.
	revisio.idVehicle := vehicle.id;
      -- Si s l'hora i data de l'entrada,
      WHEN 'entrada' THEN
        -- Recupera la hora i data.
        horaEntrada := etlNovaHora(atribut.valor);
        -- Guarda la referncia a la data i hora.
	revisio.idHoraEntrada := horaEntrada.id;
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- Guarda l'entrada a la dimensi de corresponent.
  INSERT INTO concessionariDWH.fetsRevisions
    VALUES (
      NULL,
      revisio.idBotiga,
      revisio.idMecanic,
      revisio.idClient,
      revisio.idVehicle,
      revisio.idHoraEntrada,
      NULL,
      NULL,
      0
  );
END etlEntraRevisio;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlSurtRevisio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una sortida de revisi, al magatzem.
CREATE OR REPLACE PROCEDURE etlSurtRevisio(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
  mecanic concessionariDWH.dimMecanics%rowtype;
  botiga concessionariDWH.dimBotigues%rowtype;
  client concessionariDWH.dimClients%rowtype;
  vehicle concessionariDWH.dimVehicles%rowtype;
  horaEntrada concessionariDWH.dimHores%rowtype;
  horaSortida concessionariDWH.dimHores%rowtype;
  revisio concessionariDWH.fetsRevisions%rowtype;
  revisio2 concessionariDWH.fetsRevisions%rowtype;
BEGIN
  -- Per cada atribut relacionat amb la sortida,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF del mecnic,
      WHEN 'nifMecanic' THEN
        -- recupera l'empleat,
        empleat := etlNouEmpleat(atribut.valor);
        -- el mecnic,
	mecanic := etlNouMecanic(atribut.valor);
        -- i la botiga.
	botiga := etlBotigaEmpleat(atribut.valor);
        -- Guarda el mecnic i la botiga.
	revisio.idMecanic := mecanic.id;
	revisio.idBotiga := botiga.id;
      -- Si s el NIF del client,
      WHEN 'nifClient' THEN
        -- recupera el client.
        client := etlClient(atribut.valor);
        -- Guarda el client.
	revisio.idClient := client.id;
      -- Si s el VIN del vehicle,
      WHEN 'vin' THEN
        -- recupera el vehicle.
        vehicle := etlNouVehicle(atribut.valor);
        -- Guarda el vehicle.
	revisio.idVehicle := vehicle.id;
      -- Si s la data i hora de la sortida,
      WHEN 'sortida' THEN
        -- recupera la data i hora,
        horaSortida := etlNovaHora(atribut.valor);
        -- i la guarda.
        revisio.idHoraSortida := horaSortida.id;
      -- Si s l'import,
      WHEN 'import' THEN
        -- cal guardar-lo convertint-lo a nombre.
        revisio.import := TO_NUMBER(atribut.valor,'99999999.99');
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- Recupera l'entrada.
  SELECT * INTO revisio2
    FROM concessionariDWH.fetsRevisions
    WHERE idBotiga = revisio.idBotiga
      AND idMecanic = revisio.idMecanic
      AND idClient = revisio.idClient
      AND idVehicle = revisio.idVehicle;
  -- Actualitza les dades de l'entrada
  UPDATE concessionariDWH.fetsRevisions
    SET
      idHoraSortida = revisio.idHoraSortida,
      import = revisio.import,
      tempsEmpleat = (revisio.idHoraSortida - revisio2.idHoraEntrada)*24*60*60
    WHERE idBotiga = revisio.idBotiga
      AND idMecanic = revisio.idMecanic
      AND idClient = revisio.idClient
      AND idVehicle = revisio.idVehicle;
END etlSurtRevisio;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlEntraReparacio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una entrada a reparaci, al magatzem.
CREATE OR REPLACE PROCEDURE etlEntraReparacio(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
  mecanic concessionariDWH.dimMecanics%rowtype;
  botiga concessionariDWH.dimBotigues%rowtype;
  client concessionariDWH.dimClients%rowtype;
  vehicle concessionariDWH.dimVehicles%rowtype;
  horaEntrada concessionariDWH.dimHores%rowtype;
  reparacio concessionariDWH.fetsReparacions%rowtype;
BEGIN
  -- Per cada atribut relacionat amb l'entrada,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF del mecnic,
      WHEN 'nifMecanic' THEN
        -- recupera l'empleat,
        empleat := etlNouEmpleat(atribut.valor);
        -- el mecnic,
	mecanic := etlNouMecanic(atribut.valor);
        -- i la botiga.
	botiga := etlBotigaEmpleat(atribut.valor);
        -- Guarda el mecnic i la botiga.
	reparacio.idMecanic := mecanic.id;
	reparacio.idBotiga := botiga.id;
      -- Si s el NIF del client,
      WHEN 'nifClient' THEN
        -- recupera el client.
        client := etlClient(atribut.valor);
        -- Guarda el client.
        reparacio.idClient := client.id;
      -- Si s el VIN del vehicle,
      WHEN 'vin' THEN
        -- recupera el vehicle.
        vehicle := etlNouVehicle(atribut.valor);
        -- Guarda el vehicle.
	reparacio.idVehicle := vehicle.id;
      -- Si s l'hora i data de l'entrada,
      WHEN 'entrada' THEN
        -- Recupera la hora i data.
        horaEntrada := etlNovaHora(atribut.valor);
        -- Guarda la referncia a la data i hora.
	reparacio.idHoraEntrada := horaEntrada.id;
      -- Si s el motiu,
      WHEN 'motiu' THEN
        -- es guarda.
        reparacio.motiu := atribut.valor;
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- Crea la reparaci al magatzem.
  INSERT INTO concessionariDWH.fetsReparacions
    VALUES (
        NULL,
        reparacio.idBotiga,
        reparacio.idMecanic,
        reparacio.idClient,
        reparacio.idVehicle,
        reparacio.motiu,
        reparacio.idHoraEntrada,
        NULL,
        NULL,
        0
  );
END etlEntraReparacio;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlSurtReparacio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una sortida de reparaci, al magatzem.
CREATE OR REPLACE PROCEDURE etlSurtReparacio(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
  mecanic concessionariDWH.dimMecanics%rowtype;
  botiga concessionariDWH.dimBotigues%rowtype;
  client concessionariDWH.dimClients%rowtype;
  vehicle concessionariDWH.dimVehicles%rowtype;
  horaEntrada concessionariDWH.dimHores%rowtype;
  horaSortida concessionariDWH.dimHores%rowtype;
  reparacio concessionariDWH.fetsReparacions%rowtype;
  reparacio2 concessionariDWH.fetsReparacions%rowtype;
BEGIN
  -- Per cada atribut relacionat amb la sortida,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF del mecnic,
      WHEN 'nifMecanic' THEN
        -- recupera l'empleat,
        empleat := etlNouEmpleat(atribut.valor);
        -- el mecnic,
	mecanic := etlNouMecanic(atribut.valor);
        -- i la botiga.
	botiga := etlBotigaEmpleat(atribut.valor);
        -- Guarda el mecnic i la botiga.
	reparacio.idMecanic := mecanic.id;
	reparacio.idBotiga := botiga.id;
      -- Si s el NIF del client,
      WHEN 'nifClient' THEN
        -- recupera el client.
        client := etlClient(atribut.valor);
        -- Guarda el client.
	reparacio.idClient := client.id;
      -- Si s el VIN del vehicle,
      WHEN 'vin' THEN
        -- recupera el vehicle.
        vehicle := etlNouVehicle(atribut.valor);
        -- Guarda el vehicle.
	reparacio.idVehicle := vehicle.id;
      -- Si s la data i hora de la sortida,
      WHEN 'sortida' THEN
        -- recupera la data i hora,
        horaSortida := etlNovaHora(atribut.valor);
        -- i la guarda.
        reparacio.idHoraSortida := horaSortida.id;
      -- Si s l'import,
      WHEN 'import' THEN
        -- cal guardar-lo convertint-lo a nombre.
        reparacio.import := TO_NUMBER(atribut.valor,'99999999.99');
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- Recupera la reparaci entrada.
  SELECT * INTO reparacio2
    FROM concessionariDWH.fetsReparacions
    WHERE idBotiga = reparacio.idBotiga
      AND idMecanic = reparacio.idMecanic
      AND idClient = reparacio.idClient
      AND idVehicle = reparacio.idVehicle;
  -- Actualitza la reparaci amb les dades de sortida.
  UPDATE concessionariDWH.fetsReparacions
    SET
      idHoraSortida = reparacio.idHoraSortida,
      import = reparacio.import,
      tempsEmpleat = (reparacio.idHoraSortida - reparacio2.idHoraEntrada)*24*60*60
    WHERE idBotiga = reparacio.idBotiga
      AND idMecanic = reparacio.idMecanic
      AND idClient = reparacio.idClient
      AND idVehicle = reparacio.idVehicle;
END etlSurtReparacio;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlContractacio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una contractaci, al magatzem.
CREATE OR REPLACE PROCEDURE etlContractacio(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
  mecanic concessionariDWH.dimMecanics%rowtype;
  comercial concessionariDWH.dimComercials%rowtype;
BEGIN
  -- Per cada atribut relacionat amb la contractaci,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF de l'empleat,
      WHEN 'nifEmpleat' THEN
        -- recupera l'empleat.
        empleat := etlNouEmpleat(atribut.valor);
      -- Si s el nom,
      WHEN 'nomEmpleat' THEN
        -- no cal guardar-lo.
        NULL;
      -- Si s el codi de la botiga,
      WHEN 'codiBotiga' THEN
        -- no cal guardar-lo.
        NULL;
      -- Si s el tipus d'empleat,
      WHEN 'tipusEmpleat' THEN
        -- segons el valor,
        CASE atribut.valor
          -- si s 'M', s un mecnic.
          WHEN 'M' THEN
            mecanic := etlNouMecanic(empleat.nif);
          -- Si s 'C', s un comercial.
          WHEN 'C' THEN
            comercial := etlNouComercial(empleat.nif);
          ELSE
            NULL;
        END CASE;
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
END etlContractacio;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlAcomiadament');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar un acomiadament, al magatzem.
CREATE OR REPLACE PROCEDURE etlAcomiadament(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  empleat concessionariDWH.dimEmpleats%rowtype;
BEGIN
  -- Per cada atribut relacionat amb l'acomiadamet,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el NIF de l'empleat,
      WHEN 'nifEmpleat' THEN
        -- recupera l'empleat.
        empleat := etlNouEmpleat(atribut.valor);
      -- Si s la data de la baixa,
      WHEN 'baixa' THEN
        -- actualitza la dimensi amb la data de baixa per l'empleat corresponent.
        UPDATE dimEmpleats SET baixa = TO_DATE(atribut.valor,'DD-MON-YY')
          WHERE nif = empleat.nif;
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
END etlAcomiadament;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlInauguracio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una inauguraci, al magatzem.
CREATE OR REPLACE PROCEDURE etlInauguracio(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  botiga concessionariDWH.dimBotigues%rowtype;
BEGIN
  -- Per cada atribut relacionat amb la inauguraci,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el codi de la botiga,
      WHEN 'codiBotiga' THEN
        -- transformar-lo i guardar-lo.
        botiga.codi := TO_NUMBER(atribut.valor, '9999999');
      -- Si s el nom de la botiga,
      WHEN 'nomBotiga' THEN
        -- transformar-lo i guardar-lo.
        botiga.nom := atribut.valor;
      -- Si s l'adrea,
      WHEN 'adrecaBotiga' THEN
        -- transformar-la i guardar-la.
        botiga.adreca := atribut.valor;
      -- Si s el telfon de la botiga,
      WHEN 'telfBotiga' THEN
        -- transformar-lo i guardar-lo.
        botiga.telefon := atribut.valor;
      -- Si s el fax de la botiga,
      WHEN 'faxBotiga' THEN
        -- transformar-lo i guardar-lo.
        botiga.fax := atribut.valor;
      -- Si s el correu de la botiga,
      WHEN 'correuBotiga' THEN
        -- transformar-lo i guardar-lo.
        botiga.correuE := atribut.valor;
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
  -- Crea la botiga.
  INSERT INTO dimBotigues
    VALUES (
      NULL,
      botiga.codi,
      botiga.nom,
      botiga.adreca,
      botiga.telefon,
      botiga.fax,
      botiga.correuE,
      NULL
  );
END etlInauguracio;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etlClausura');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment per transformar i traspassar una clausura, al magatzem.
CREATE OR REPLACE PROCEDURE etlClausura(registre IN concessionari.registres%rowtype) IS
  CURSOR atributs1(codiR IN NUMBER) IS SELECT * FROM concessionari.atributs WHERE codiRegistre = codiR;
  botiga concessionariDWH.dimBotigues%rowtype;
BEGIN
  -- Per cada atribut relacionat amb la clausura,
  FOR atribut IN atributs1(registre.codi) LOOP
  BEGIN
    -- segons el nom de l'atribut,
    CASE atribut.nom
      -- si s el codi de la botiga clausurada,
      WHEN 'codiClausurada' THEN
        -- es recupera la botiga.
        SELECT * into botiga from dimBotigues WHERE codi = TO_NUMBER(atribut.valor,'9999999');
      -- Si s el codi de la botiga on van els treballadors,
      WHEN 'codiNova' THEN
        -- no es fa res.
        NULL;
      -- Si s la data de clausura,
      WHEN 'baixa' THEN
        -- s'actualitza el camp de la data de clausura, amb el valor transformat.
        UPDATE dimBotigues SET clausurada = TO_DATE(atribut.valor, 'DD-MON-YY') WHERE codi = botiga.codi;
      -- Si no s cap atribut reconegut,
      ELSE
        -- es mostra un missatge amb la informaci que es t.
        DBMS_OUTPUT.PUT_LINE(
          'Atribut desconegut ' || 
          atribut.nom || '(' || 
          atribut.tipus || ') := ' || 
          atribut.valor
        );
    END CASE;
    END;
  END LOOP;
END etlClausura;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment etl');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Procediment principal d'ETL.
CREATE OR REPLACE PROCEDURE etl IS
  ultimEtl NUMBER := 0;
  nifEmpleat VARCHAR2(9);
  CURSOR registres1(ultimCodi IN NUMBER) IS 
    SELECT * FROM concessionari.registres WHERE codi > ultimCodi AND nomFuncio != 'etl';
BEGIN
  -- Troba la ltim execuci de l'ETL.
  SELECT MAX(codi) INTO ultimEtl FROM concessionari.registres WHERE nomFuncio = 'etl';
  -- Si no hi ha cap execuci prvia, 
  IF ultimEtl IS NULL THEN
    -- el primer registre s l'ltim ETL.
    ultimEtl:=0;
  END IF;
  -- Per cada registre des de l'ltim ETL, 
  FOR registre IN registres1(ultimEtl) LOOP
  BEGIN
    -- segons el nombre de la funci,
    CASE registre.nomFuncio
      -- si s una compra,
      WHEN 'compra' THEN
        -- executa l'etl de la compra.
        etlCompra(registre);
      -- Si s un nou venedor,
      WHEN 'nouVenedor' THEN
        -- no cal fer res.
        NULL;
      -- Si s un nou venda,
      WHEN 'venda' THEN
        -- executa l'etl de la venda.
        etlVenda(registre);
      -- Si s un nou client,
      WHEN 'nouClient' THEN
        -- executa l'etl del nou client.
        etlNouClient(registre);
      -- Si s una entrada a revisi,
      WHEN 'entraRevisio' THEN
        -- executa l'etl de l'entrada a revisi.
        etlEntraRevisio(registre);
      -- Si s una sortida a revisi,
      WHEN 'surtRevisio' THEN
        -- executa l'etl de la sortida de revisi.
        etlSurtRevisio(registre);
      -- Si s una entrada a reparaci,
      WHEN 'entraReparacio' THEN
        -- executa l'etl de l'entrada a reparaci.
        etlEntraReparacio(registre);
      -- Si s una sortida de reparaci,
      WHEN 'surtReparacio' THEN
        -- executa l'etl de la sortida de reparaci.
        etlSurtReparacio(registre);
      -- Si s una contractaci,
      WHEN 'contractacio' THEN
        -- executa l'etl de la contractaci.
        etlContractacio(registre);
      -- Si s un acomiadament,
      WHEN 'acomiadament' THEN
        -- executa l'etl de l'acomiadament.
        etlAcomiadament(registre);
      -- Si s una inauguraci,
      WHEN 'inauguracio' THEN
        -- executa l'etl de la inauguraci.
        etlInauguracio(registre);
      -- Si s una clausura,
      WHEN 'clausura' THEN
        -- executa l'etl de la clausura.
        etlClausura(registre);
      -- Si no es reconeix la funci,
      ELSE
        -- mostra un missatge amb tota la informaci que es t.
        DBMS_OUTPUT.PUT_LINE('Funcio ' || registre.nomFuncio || ' desconeguda');
    END CASE;
    -- Si no hi ha registres, cal sortir.
    EXIT WHEN registres1%NOTFOUND;
  END;
  END LOOP;
  -- Registra l'execuci de l'ETL.
  concessionari.registra(
    'etl',
    concessionari.tAtributs(
      concessionari.atribut(
        'execucio',
        TO_CHAR(CURRENT_TIMESTAMP,'YYYYMMDDHHmmSS'),
        'timestamp'
      )
    )
  );
END etl;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET SERVEROUTPUT OFF
SET TERMOUT OFF