SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('Creant tipus de dades pel subsistema de registre');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el tipus atribut que representar un atribut del registre.
-- Consta de tres valors: el nom de l'atribut, el seu valor en format
-- de cadena i el tipus real de la dada.
CREATE OR REPLACE TYPE atribut IS OBJECT (
  nom VARCHAR2(100),
  valor VARCHAR2(200),
  tipus VARCHAR2(200)
);
/

-- Crear el tipus tAtributs que representar tots els atributs d'un
-- mateix registre, en un vector.
CREATE OR REPLACE TYPE tAtributs IS VARRAY(100) OF atribut;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('Creant procediments operatius');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment registra');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per registrar events
CREATE OR REPLACE PROCEDURE registra (nomFuncio IN VARCHAR2, v_tAtributs IN tAtributs) IS
BEGIN
  -- Crea el nou registre, associant-hi el nom de la funci.
  INSERT INTO registres VALUES (NULL,nomFuncio,CURRENT_TIMESTAMP);
  -- Inserta cada atribut especificat als parmetres, associant-lo al registre recent creat.
  FOR i IN v_tAtributs.first..v_tAtributs.last
  LOOP
    INSERT INTO atributs VALUES (
      registres_codi.currval,
      v_tAtributs(i).nom,
      v_tAtributs(i).valor,
      v_tAtributs(i).tipus
    );
  END LOOP;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment nouVenedor');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per crear nous venedors.
CREATE OR REPLACE PROCEDURE nouVenedor (
  nif IN VARCHAR2, 
  nom IN VARCHAR2, 
  adreca IN VARCHAR2, 
  telefon IN VARCHAR2, 
  fax IN VARCHAR2, 
  correuE IN VARCHAR2
) IS
BEGIN
  -- Crea el nou venedor.
  INSERT INTO venedors VALUES (nif, nom, adreca, telefon, fax, correuE);
  -- Registra la creaci del nou venedor.
  registra('nouVenedor',tAtributs(
    atribut('nif',nif,'VARCHAR2'),
    atribut('nom',nom,'VARCHAR2'),
    atribut('adreca',adreca,'VARCHAR2'),
    atribut('telefon',telefon,'VARCHAR2'),
    atribut('fax',fax,'VARCHAR2'),
    atribut('correuE',correuE,'VARCHAR2')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment compra');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per realitzar compres.
CREATE OR REPLACE PROCEDURE compra (
  nifEmpleat IN VARCHAR2,
  nifVenedor IN VARCHAR2,
  vin IN VARCHAR2,
  matricula IN VARCHAR2,
  marca IN VARCHAR2,
  modelCotxe IN VARCHAR2,
  motor IN VARCHAR2,
  equipament IN VARCHAR2,
  color IN VARCHAR2,
  dataCompra IN DATE,
  importCompra IN NUMBER,
  kms IN NUMBER DEFAULT 0
) IS
  empleat empleats%rowtype;
BEGIN
  -- Verifica si l'empleat que fa la compra existeix i est en actiu.
  SELECT * INTO empleat FROM empleats WHERE nif = nifEmpleat;
  -- Si l'empleat no est en actiu, cal generar l'error corresponent.
  IF empleat.enActiu = 'N' THEN
    raise_application_error(-20101, empleat.nom || ' (NIF:' || empleat.nif ||') ja no treballa aqui');
  END IF;
  -- Inserir el nou vehicle
  INSERT INTO vehicles VALUES (vin, marca, modelCotxe, motor, equipament, color, kms);
  -- Si el vehcile t una matrcula, es crea l'associaci corresponent.
  IF NOT matricula IS NULL THEN
    INSERT INTO matricules VALUES (matricula, vin);
  END IF;
  -- S'inserta la compra.
  INSERT INTO compres VALUES (nifEmpleat, nifVenedor, vin, dataCompra, importCompra);
  -- Es registra la compra.
  registra('compra',tAtributs(
    atribut('nifEmpleat',nifEmpleat,'VARCHAR2'),
    atribut('nifVenedor',nifVenedor,'VARCHAR2'),
    atribut('vin',vin,'VARCHAR2'),
    atribut('data',dataCompra,'DATE'),
    atribut('import',importCompra,'NUMBER')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment nouClient');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per crear nous clients.
CREATE OR REPLACE PROCEDURE nouClient (
  nif IN VARCHAR2,
  nom IN VARCHAR2,
  adreca IN VARCHAR2,
  telefon IN VARCHAR2,
  fax IN VARCHAR2,
  correuE IN VARCHAR2
) IS
BEGIN
  -- Crea el nou client.
  INSERT INTO clients VALUES (nif, nom, adreca, telefon, fax, correuE);
  -- Registra la creaci del nou client.
  registra('nouClient',tAtributs(
    atribut('nif',nif,'VARCHAR2'),
    atribut('nom',nom,'VARCHAR2'),
    atribut('adreca',adreca,'VARCHAR2'),
    atribut('telefon',telefon,'VARCHAR2'),
    atribut('fax',fax,'VARCHAR2'),
    atribut('correuE',correuE,'VARCHAR2')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment venda');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per realitzar compres.
CREATE OR REPLACE PROCEDURE venda (
  nifComercial IN VARCHAR2,
  nifClient IN VARCHAR2,
  vin IN VARCHAR2,
  dataVenda IN DATE,
  importVenda IN NUMBER
) IS
  matricules INT := 0;
  matricula VARCHAR2(7);
  novaMatricula VARCHAR2(7);
  empleat empleats%rowtype;
BEGIN
  -- Identifica si el comercial existeix i est en actiu
  SELECT * INTO empleat FROM empleats WHERE nif = nifComercial;
  -- Si el comercial no est en actiu, genera l'error corresponent.
  IF empleat.enActiu = 'N' THEN
    raise_application_error(-20101, empleat.nom || ' (NIF:' || empleat.nif ||') ja no treballa aqui');
  END IF;
  -- Crea la venda.
  INSERT INTO vendes VALUES (nifComercial, nifClient, vin, dataVenda, importVenda);
  -- Registra la venda.
  registra('venda',tAtributs(
    atribut('nifComercial',nifComercial,'VARCHAR2'),
    atribut('nifClient',nifClient,'VARCHAR2'),
    atribut('vin',vin,'VARCHAR2'),
    atribut('data',dataVenda,'DATE'),
    atribut('import',importVenda,'NUMBER')
  ));
  -- Recupera la quantitat de matrcules associades al vehicle.
  SELECT COUNT(matricula) INTO matricules FROM matricules WHERE vinVehicle = vin;
  -- Si el vehicle no est matriculat,
  IF matricules = 0 THEN
    -- Recupera la quantitat de matricules sense associar a cap vehicle.
    SELECT COUNT(matricula) INTO matricules FROM matricules WHERE vinVehicle IS NULL;
    -- Si hi ha matrcules lliures, 
    IF matricules > 0 THEN
      -- Recupera una matrcula lliure.
      SELECT matricula INTO novaMatricula FROM matricules WHERE vinVehicle IS NULL;
      -- Associa la matrcula lliure al vehicle que es ven.
      UPDATE matricules SET vinVehicle = vin WHERE matricula = novaMatricula;
    ELSE
      RAISE NO_DATA_FOUND;
    END IF;
  END IF;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment novaMatricula');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear procediment per afegir una nova matrcula.
CREATE OR REPLACE PROCEDURE novaMatricula (
  matricula IN VARCHAR2
) IS
BEGIN
  -- Crea la nova matrcula.
  INSERT INTO matricules VALUES (matricula, NULL);
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment entraRevisio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per entrar vehicles per revisio.
CREATE OR REPLACE PROCEDURE entraRevisio (
  nifMecanic IN VARCHAR2,
  nifClient IN VARCHAR2,
  vinVehicle IN VARCHAR2,
  matricula IN VARCHAR2,
  model IN VARCHAR2,
  marca IN VARCHAR2,
  motor IN VARCHAR2,
  equipament IN VARCHAR2,
  color IN VARCHAR2,
  kmsNous IN NUMBER
) IS
  vehicle vehicles%rowtype;
  entrada TIMESTAMP(9);
  empleat empleats%rowtype;
BEGIN
  -- Comprova si el mecnic existeix i est en actiu.
  SELECT * INTO empleat FROM empleats WHERE nif = nifMecanic;
  -- Si no est en actiu, genera l'error corresponent.
  IF empleat.enActiu = 'N' THEN
    raise_application_error(-20101, empleat.nom || ' (NIF:' || empleat.nif ||') ja no treballa aqui');
  END IF;
  -- L'hora d'entrada, ser en la que s'executi el procediment.
  entrada := CURRENT_TIMESTAMP;
  -- Comprova si existeix el vehicle.
  SELECT * INTO vehicle FROM vehicles WHERE vin = vinVehicle;
  -- Crea la revisi.
  INSERT INTO revisions VALUES(nifMecanic,nifClient,vinVehicle,entrada,NULL,0);
  -- Actualitza les dades del vehicle.
  UPDATE vehicles SET kms = kmsNous WHERE vin = vinVehicle;
  -- Registra l'entrada a revisi.
  registra('entraRevisio',tAtributs(
    atribut('nifMecanic',nifMecanic,'VARCHAR2'),
    atribut('nifClient',nifClient,'VARCHAR2'),
    atribut('vin',vinVehicle,'VARCHAR2'),
    atribut('entrada',entrada,'DATE')
  ));
EXCEPTION
  -- En cas que no es trobi el vehicle
  WHEN NO_DATA_FOUND THEN
    -- Crea el vehicle.
    INSERT INTO vehicles VALUES(vinVehicle,marca,model,motor,equipament,color,kmsNous);
    -- Registra la matrcula.
    INSERT INTO matricules VALUES(matricula,vinVehicle);
    -- Crea la revisi.
    INSERT INTO revisions VALUES(nifMecanic,nifClient,vinVehicle,entrada,NULL,0);
    -- Registra l'entrada a revisi.
    registra('entraRevisio',tAtributs(
      atribut('nifMecanic',nifMecanic,'VARCHAR2'),
      atribut('nifClient',nifClient,'VARCHAR2'),
      atribut('vin',vinVehicle,'VARCHAR2'),
      atribut('entrada',entrada,'DATE')
    ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment surtRevisio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per sortir de revisi.
CREATE OR REPLACE PROCEDURE surtRevisio (
  nifMecanic IN VARCHAR2,
  nifClient IN VARCHAR2,
  vinVehicle IN VARCHAR2,
  importNou IN NUMBER
) IS
  sortida2 TIMESTAMP(9);
  empleat empleats%rowtype;
BEGIN
  -- Comprova si el mecnic existeix i est en actiu.
  SELECT * INTO empleat FROM empleats WHERE nif = nifMecanic;
  -- Si no est en actiu, genera l'error corresponent.
  IF empleat.enActiu = 'N' THEN
    raise_application_error(-20101, empleat.nom || ' (NIF:' || empleat.nif ||') ja no treballa aqui');
  END IF;
  -- L'hora de sortida es calcula en moment d'executar el procediment.
  sortida2 := CURRENT_TIMESTAMP;
  -- Canvia la revisi per guardar la sortida.
  UPDATE revisions
    SET
      sortida = sortida2,
      import = importNou
    WHERE nifMecanic = nifMecanic
      AND nifClient = nifClient
      AND vinVehicle = vinVehicle
      AND import = 0;
  -- Registra la sortida de la revisi.
  registra('surtRevisio',tAtributs(
    atribut('nifMecanic',nifMecanic,'VARCHAR2'),
    atribut('nifClient',nifClient,'VARCHAR2'),
    atribut('vin',vinVehicle,'VARCHAR2'),
    atribut('sortida',sortida2,'DATE'),
    atribut('import',importNou,'NUMBER')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment entraReparacio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per entrar a reparar.
CREATE OR REPLACE PROCEDURE entraReparacio (
  nifMecanic IN VARCHAR2,
  nifClient IN VARCHAR2,
  vinVehicle IN VARCHAR2,
  matricula IN VARCHAR2,
  model IN VARCHAR2,
  marca IN VARCHAR2,
  motor IN VARCHAR2,
  equipament IN VARCHAR2,
  color IN VARCHAR2,
  kmsNous IN NUMBER,
  motiu IN VARCHAR2
) IS
  vehicle vehicles%rowtype;
  entrada TIMESTAMP(9);
  empleat empleats%rowtype;
BEGIN
  -- Comprova si el mecnic existeix i est en actiu.
  SELECT * INTO empleat FROM empleats WHERE nif = nifMecanic;
  -- Si no est en actiu, genera l'error corresponent.
  IF empleat.enActiu = 'N' THEN
    raise_application_error(-20101, empleat.nom || ' (NIF:' || empleat.nif ||') ja no treballa aqui');
  END IF;
  -- L'hora d'entrada, ser en la que s'executi el procediment.
  entrada := CURRENT_TIMESTAMP;
  -- Comprova si existeix el vehicle.
  SELECT * INTO vehicle FROM vehicles WHERE vin = vinVehicle;
  -- Crea la reparaci.
  INSERT INTO reparacions VALUES(nifMecanic,nifClient,vinVehicle,motiu,entrada,NULL,0);
  -- Actualitza les dades del vehicle.
  UPDATE vehicles SET kms = kmsNous WHERE vin = vinVehicle;
  -- Registra l'entrada a revisi.
  registra('entraReparacio',tAtributs(
    atribut('nifMecanic',nifMecanic,'VARCHAR2'),
    atribut('nifClient',nifClient,'VARCHAR2'),
    atribut('vin',vinVehicle,'VARCHAR2'),
    atribut('entrada',entrada,'DATE'),
    atribut('motiu',motiu,'VARCHAR2')
  ));
EXCEPTION
  -- En cas que no es trobi el vehicle
  WHEN NO_DATA_FOUND THEN
    -- Crea el vehicle.
    INSERT INTO vehicles VALUES(vinVehicle,marca,model,motor,equipament,color,kmsNous);
    -- Registra la matrcula.
    INSERT INTO matricules VALUES(matricula,vinVehicle);
    -- Crea la revisi.
    INSERT INTO reparacions VALUES(nifMecanic,nifClient,vinVehicle,motiu,entrada,NULL,0);
    -- Registra l'entrada a revisi.
    registra('entraReparacio',tAtributs(
      atribut('nifMecanic',nifMecanic,'VARCHAR2'),
      atribut('nifClient',nifClient,'VARCHAR2'),
      atribut('vin',vinVehicle,'VARCHAR2'),
      atribut('entrada',entrada,'DATE'),
      atribut('motiu',motiu,'VARCHAR2')
    ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment surtReparacio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per sortir de reparaci.
CREATE OR REPLACE PROCEDURE surtReparacio (
  nifMecanic IN VARCHAR2,
  nifClient IN VARCHAR2,
  vinVehicle IN VARCHAR2,
  importNou IN NUMBER
) IS
  sortida2 TIMESTAMP(9);
  empleat empleats%rowtype;
BEGIN
  -- Comprova si el mecnic existeix i est en actiu.
  SELECT * INTO empleat FROM empleats WHERE nif = nifMecanic;
  -- Si no est en actiu, genera l'error corresponent.
  IF empleat.enActiu = 'N' THEN
    raise_application_error(-20101, empleat.nom || ' (NIF:' || empleat.nif ||') ja no treballa aqui');
  END IF;
  -- L'hora de sortida es calcula en moment d'executar el procediment.
  sortida2 := CURRENT_TIMESTAMP;
  -- Canvia la revisi per guardar la sortida.
  UPDATE reparacions
    SET
      sortida = sortida2,
      import = importNou
    WHERE nifMecanic = nifMecanic
      AND nifClient = nifClient
      AND vinVehicle = vinVehicle
      AND import = 0;
  registra('surtReparacio',tAtributs(
    atribut('nifMecanic',nifMecanic,'VARCHAR2'),
    atribut('nifClient',nifClient,'VARCHAR2'),
    atribut('vin',vinVehicle,'VARCHAR2'),
    atribut('sortida',sortida2,'DATE'),
    atribut('import',importNou,'NUMBER')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment contractacio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per fer contractacions.
CREATE OR REPLACE PROCEDURE contractacio (
  nifEmpleat IN VARCHAR2,
  nomEmpleat IN VARCHAR2,
  codiBotiga IN NUMBER,
  tipusEmpleat IN VARCHAR2
) IS
BEGIN
  -- Crea el nou empleat.
  INSERT INTO empleats VALUES (nifEmpleat, nomEmpleat, codiBotiga, 'Y');
  -- Segons el tipus d'empleat,
  CASE tipusEmpleat
    -- si s un mecnic,
    WHEN 'M' THEN
      -- es crea un mecnic;
      INSERT INTO mecanics VALUES (nifEmpleat);
    -- si s un comercial,
    WHEN 'C' THEN
      -- es crea un comercial.
      INSERT INTO comercials VALUES (nifEmpleat);
  END CASE;
  -- Registra la contractaci.
  registra('contractacio',tAtributs(
    atribut('nifEmpleat',nifEmpleat,'VARCHAR2'),
    atribut('nomEmpleat',nomEmpleat,'VARCHAR2'),
    atribut('codiBotiga',codiBotiga,'VARCHAR2'),
    atribut('tipusEmpleat',tipusEmpleat,'VARCHAR2')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment acomiadament');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear procediment acomiadament.
CREATE OR REPLACE PROCEDURE acomiadament(nifEmpleat IN VARCHAR2) IS
BEGIN
  -- Modifica el registra de l'empleat, canviant el camp enActiu.
  UPDATE empleats SET enActiu = 'N' WHERE empleats.nif = nifEmpleat;
  -- Registra l'acomiadament.
  registra('acomiadament',tAtributs(
    atribut('nifEmpleat',nifEmpleat,'VARCHAR2'),
    atribut('baixa',CURRENT_DATE,'DATE')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment inauguracio');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment d'inauguraci
CREATE OR REPLACE PROCEDURE inauguracio(
  nomBotiga IN VARCHAR2,
  adrecaBotiga IN VARCHAR2,
  telfBotiga IN VARCHAR2,
  faxBotiga IN VARCHAR2,
  correuBotiga IN VARCHAR2
) IS
  codiBotiga int;
BEGIN
  -- Crea la botiga.
  INSERT INTO botigues VALUES(
    null,
    nomBotiga,
    adrecaBotiga,
    telfBotiga,
    faxBotiga,
    correuBotiga
  );
  -- Obt el codi de la nova botiga.
  SELECT MAX(codi) into codiBotiga from botigues;
  -- Registra la inauguraci.
  registra('inauguracio',tAtributs(
    atribut('codiBotiga', codiBotiga, 'NUMBER'),
    atribut('nomBotiga', nomBotiga, 'VARCHAR2'),
    atribut('adrecaBotiga', adrecaBotiga, 'VARCHAR2'),
    atribut('telfBotiga', telfBotiga, 'VARCHAR2'),
    atribut('faxBotiga', faxBotiga, 'VARCHAR2'),
    atribut('correuBotiga', correuBotiga, 'VARCHAR2')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE(' Procediment clausura');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per la clausura.
CREATE OR REPLACE PROCEDURE clausura(
  codiClausurada IN NUMBER,
  codiNova IN NUMBER
) IS
BEGIN
  -- Assigna tots els empleats de la botiga clausurada a la nova.
  UPDATE empleats SET codiBotiga = codiNova WHERE codiBotiga = codiClausurada;
  -- Esborra la botiga clausurada.
  DELETE FROM botigues WHERE codi = codiClausurada;
  -- Registra la clausura de la tenda.
  registra('clausura',tAtributs(
    atribut('codiClausurada', codiClausurada, 'NUMBER'),
    atribut('codiNova', codiNova, 'VARCHAR2'),
    atribut('baixa',CURRENT_DATE,'DATE')
  ));
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF