SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('Creant taules per la base de dades');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula botigues');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de botigues
CREATE TABLE botigues (
  codi NUMBER PRIMARY KEY,
  nom VARCHAR2(100) NOT NULL,
  adreca VARCHAR2(200) NOT NULL,
  telefon VARCHAR2(9) NOT NULL,
  fax VARCHAR2(9) NOT NULL,
  correuE VARCHAR2(50) NOT NULL
);

-- Crear la seqncia pel codi nic de les botigues
CREATE SEQUENCE botigues_codi
  START WITH 1
  INCREMENT BY 1;

-- Crear el disparador per substituir el valor que es proveeixi
-- pel codi en insertar una botiga pel segent nombre de la
-- seqncia corresponent.
CREATE OR REPLACE TRIGGER botigues_codi_trigger
BEFORE INSERT
ON botigues
REFERENCING NEW AS NEW
FOR EACH ROW
BEGIN
  SELECT botigues_codi.nextval INTO :NEW.codi FROM dual;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula empleats');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula d'empleats
CREATE TABLE empleats (
  nif VARCHAR2(9) PRIMARY KEY,
  nom VARCHAR2(200) NOT NULL,
  codiBotiga NUMBER NOT NULL,
  enActiu CHAR(1) DEFAULT 'Y' NOT NULL CHECK(enActiu IN ('Y','N')),
  CONSTRAINT fk_botiga_empleats FOREIGN KEY (codiBotiga) REFERENCES botigues(codi)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula vehicles');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de vehicles
CREATE TABLE vehicles (
  vin VARCHAR2(17) PRIMARY KEY,
  marca VARCHAR2(30) NOT NULL,
  model VARCHAR2(50) NOT NULL,
  motor VARCHAR2(50) NOT NULL,
  equipament VARCHAR2(200) NOT NULL,
  color VARCHAR2(20) NOT NULL,
  kms NUMBER(8) DEFAULT 0 NOT NULL
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula venedors');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de venedors
CREATE TABLE venedors (
  nif VARCHAR2(9) PRIMARY KEY,
  nom VARCHAR2(100) NOT NULL,
  adreca VARCHAR2(200) NOT NULL,
  telefon VARCHAR2(9) NOT NULL,
  fax VARCHAR2(9) NOT NULL,
  correuE VARCHAR2(50) NOT NULL
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula compres');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de compres
CREATE TABLE compres (
  nifEmpleat VARCHAR2(9) NOT NULL,
  nifVenedor VARCHAR2(9) NOT NULL,
  vinVehicle VARCHAR2(17) NOT NULL,
  data DATE NOT NULL,
  import NUMBER(10,2) NOT NULL,
  CONSTRAINT fk_empleat_compres FOREIGN KEY (nifEmpleat) REFERENCES empleats(nif),
  CONSTRAINT fk_venedor_compres FOREIGN KEY (nifVenedor) REFERENCES venedors(nif),
  CONSTRAINT fk_vehicle_compres FOREIGN KEY (vinVehicle) REFERENCES vehicles(vin),
  CONSTRAINT pk_compres PRIMARY KEY (nifEmpleat,nifVenedor,vinVehicle,data)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula comercials');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de comercials
CREATE TABLE comercials (
  nifEmpleat VARCHAR2(9) PRIMARY KEY,
  CONSTRAINT fk_empleat_comercials FOREIGN KEY (nifEmpleat) REFERENCES empleats(nif)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula clients');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de clients
CREATE TABLE clients (
  nif VARCHAR2(9) PRIMARY KEY,
  nom VARCHAR2(100) NOT NULL,
  adreca VARCHAR2(200) NOT NULL,
  telefon VARCHAR2(9) NOT NULL,
  fax VARCHAR2(9) NOT NULL,
  correuE VARCHAR2(50) NOT NULL
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula vendes');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de vendes
CREATE TABLE vendes (
  nifComercial VARCHAR2(9) NOT NULL,
  nifClient VARCHAR2(9) NOT NULL,
  vinVehicle VARCHAR2(17) NOT NULL,
  data DATE NOT NULL,
  import NUMBER(10,2) NOT NULL,
  CONSTRAINT fk_comercial_vendes FOREIGN KEY (nifComercial) REFERENCES comercials(nifEmpleat),
  CONSTRAINT fk_client_vendes FOREIGN KEY (nifClient) REFERENCES clients(nif),
  CONSTRAINT fk_vehicle_vendes FOREIGN KEY (vinVehicle) REFERENCES vehicles(vin),
  CONSTRAINT pk_vendes PRIMARY KEY (nifComercial,nifClient,vinVehicle,data)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula matricules');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de matricules
CREATE TABLE matricules (
  matricula VARCHAR2(7) PRIMARY KEY,
  vinVehicle VARCHAR2(17),
  CONSTRAINT fk_vehicle_matricules FOREIGN KEY (vinVehicle) REFERENCES vehicles(vin)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula mecanics');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de mecanics
CREATE TABLE mecanics (
  nifEmpleat VARCHAR2(9) PRIMARY KEY,
  CONSTRAINT fk_empleat_mecanics FOREIGN KEY (nifEmpleat) REFERENCES empleats(nif)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula revisions');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de revisions
CREATE TABLE revisions (
  nifMecanic VARCHAR2(9) NOT NULL,
  nifClient VARCHAR2(9) NOT NULL,
  vinVehicle VARCHAR2(17) NOT NULL,
  entrada TIMESTAMP(9) NOT NULL,
  sortida TIMESTAMP(9),
  import NUMBER(10,2) NOT NULL,
  CONSTRAINT fk_mecanic_revisions FOREIGN KEY (nifMecanic) REFERENCES mecanics(nifEmpleat),
  CONSTRAINT fk_client_revisions FOREIGN KEY (nifClient) REFERENCES clients(nif),
  CONSTRAINT fk_vehicle_revisions FOREIGN KEY (vinVehicle) REFERENCES vehicles(vin),
  CONSTRAINT pk_revisions PRIMARY KEY (nifMecanic,nifClient,vinVehicle,entrada)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula reparacions');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de reparacions
CREATE TABLE reparacions (
  nifMecanic VARCHAR2(9) NOT NULL,
  nifClient VARCHAR2(9) NOT NULL,
  vinVehicle VARCHAR2(17) NOT NULL,
  motiu VARCHAR2(200) NOT NULL,
  entrada TIMESTAMP(9) NOT NULL,
  sortida TIMESTAMP(9),
  import NUMBER(10,2) NOT NULL,
  CONSTRAINT fk_mecanic_reparacions FOREIGN KEY (nifMecanic) REFERENCES mecanics(nifEmpleat),
  CONSTRAINT fk_client_reparacions FOREIGN KEY (nifClient) REFERENCES clients(nif),
  CONSTRAINT fk_vehicle_reparacions FOREIGN KEY (vinVehicle) REFERENCES vehicles(vin),
  CONSTRAINT pk_reparacions PRIMARY KEY (nifMecanic,nifClient,vinVehicle,entrada)
);

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula registres');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula de registres
CREATE TABLE registres (
  codi NUMBER PRIMARY KEY,
  nomFuncio VARCHAR2(100) NOT NULL,
  data TIMESTAMP NOT NULL
);

-- Crear la seqncia pel codi nic de les botigues
CREATE SEQUENCE registres_codi
  START WITH 1
  INCREMENT BY 1;

-- Crear el disparador per substituir el valor que es proveeixi
-- pel codi en insertar una botiga pel segent nombre de la
-- seqncia corresponent.
CREATE OR REPLACE TRIGGER registres_codi_trigger
BEFORE INSERT
ON registres
REFERENCING NEW AS NEW
FOR EACH ROW
BEGIN
  SELECT registres_codi.nextval INTO :NEW.codi FROM dual;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE(' Taula atributs');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear la taula d'atributs
CREATE TABLE atributs (
  codiRegistre NUMBER NOT NULL,
  nom VARCHAR2(100) NOT NULL,
  valor VARCHAR2(200) NOT NULL,
  tipus VARCHAR2(200) NOT NULL,
  CONSTRAINT fk_registre_atributs FOREIGN KEY (codiRegistre) REFERENCES registres(codi),
  CONSTRAINT pk_atributs PRIMARY KEY (codiRegistre,nom)
);