SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 101: Contractaci i acomiadaments.');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la contractaci i l'acomiadament.
CREATE OR REPLACE TYPE tasca101_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE setup_testing,
  MEMBER PROCEDURE test_contractacio,
  MEMBER PROCEDURE test_acomiadament,
  MEMBER PROCEDURE test_acomiadament_compra,
  MEMBER PROCEDURE test_acomiadament_venda,
  MEMBER PROCEDURE test_acomiadament_erevisio,
  MEMBER PROCEDURE test_acomiadament_srevisio,
  MEMBER PROCEDURE test_acomiadament_ereparacio,
  MEMBER PROCEDURE test_acomiadament_sreparacio,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca101_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos de les validacions de la contractaci i l'acomiadament.
CREATE OR REPLACE TYPE BODY tasca101_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE setup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO empleats VALUES (
      '11102989E',
      'Marc',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102989E'
    );
    INSERT INTO mecanics VALUES (
      '11102989E'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
    INSERT INTO clients VALUES (
      '90909090D',
      'Manel',
      'c. Vigo',
      '986574833',
      '986574834',
      'manel@gmail.com'
    );
  END setup_testing;

  -- Aquest test prova la contractaci d'un mecnic i d'un comercial.
  MEMBER PROCEDURE test_contractacio IS
    n_empleats    INT := 0;
    n_comercials  INT := 0;
    n_mecanics    INT := 0;
    n_empleats2   INT := 0;
    n_comercials2 INT := 0;
    n_mecanics2   INT := 0;
    codi_botiga   INT;
  BEGIN
    SELECT MAX(codi) INTO codi_botiga FROM botigues;
    SELECT COUNT(*) INTO n_empleats FROM empleats;
    SELECT COUNT(*) INTO n_mecanics FROM mecanics;
    SELECT COUNT(*) INTO n_comercials FROM comercials;
    contractacio('12102989E','Alex',codi_botiga,'M');
    SELECT COUNT(*) INTO n_empleats2 FROM empleats;
    SELECT COUNT(*) INTO n_mecanics2 FROM mecanics;
    SELECT COUNT(*) INTO n_comercials2 FROM comercials;
    m_util_object.ok(
      test_passed => (
        n_empleats2 = n_empleats + 1 AND
        n_mecanics2 = n_mecanics + 1 AND 
        n_comercials2 = n_comercials
      ),
      test_label  => 'Contractar un mecnic'
    );
    contractacio('13102989E','Carles',codi_botiga,'C');
    SELECT COUNT(*) INTO n_empleats2 FROM empleats;
    SELECT COUNT(*) INTO n_mecanics2 FROM mecanics;
    SELECT COUNT(*) INTO n_comercials2 FROM comercials;
    m_util_object.ok(
      test_passed => (
        n_empleats2 = n_empleats + 2 AND
        n_mecanics2 = n_mecanics + 1 AND 
        n_comercials2 = n_comercials + 1
      ),
      test_label  => 'Contractar un comercial'
    );
  END test_contractacio;

  -- Aquest test contracta un mecnic i un comercial i els acomiada.
  MEMBER PROCEDURE test_acomiadament IS
    n_empleats    INT := 0;
    n_empleats2   INT := 0;
    codi_botiga   INT;
  BEGIN
    SELECT MAX(codi) INTO codi_botiga FROM botigues;
    contractacio('12102989E','Alex',codi_botiga,'M');
    contractacio('13102989E','Carles',codi_botiga,'C');
    SELECT COUNT(*) INTO n_empleats FROM empleats WHERE enActiu = 'Y';
    acomiadament('12102989E');
    SELECT COUNT(*) INTO n_empleats2 FROM empleats WHERE enActiu = 'Y';
    m_util_object.ok(
      test_passed => ( n_empleats2 = n_empleats - 1 ),
      test_label  => 'Acomiadar un mecnic'
    );
    SELECT COUNT(*) INTO n_empleats FROM empleats WHERE enActiu = 'Y';
    acomiadament('13102989E');
    SELECT COUNT(*) INTO n_empleats2 FROM empleats WHERE enActiu = 'Y';
    m_util_object.ok(
      test_passed => ( n_empleats2 = n_empleats -1 ),
      test_label  => 'Acomiadar un comercial'
    );
  END test_acomiadament;

  -- Aquest test verifica l'error en fer una compra per un empleat acomiadat.
  MEMBER PROCEDURE test_acomiadament_compra IS
    empleat_no_actiu EXCEPTION;
    PRAGMA EXCEPTION_INIT(empleat_no_actiu, -20101);
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    acomiadament('10102989E');
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9799D',
      '4337HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    m_util_object.ok(
      test_passed => FALSE,
      test_label  => 'Empleat acomiadat (compra)'
    );
  EXCEPTION
    WHEN empleat_no_actiu THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label  => 'Empleat acomiadat (compra)'
      );
  END test_acomiadament_compra;

  -- Aquest test verifica l'error en fer una venda per un empleat acomiadat.
  MEMBER PROCEDURE test_acomiadament_venda IS
    empleat_no_actiu EXCEPTION;
    PRAGMA EXCEPTION_INIT(empleat_no_actiu, -20101);
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    acomiadament('10102989E');
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    m_util_object.ok(
      test_passed => FALSE,
      test_label  => 'Empleat acomiadat (venda)'
    );
  EXCEPTION
    WHEN empleat_no_actiu THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label  => 'Empleat acomiadat (venda)'
      );
  END test_acomiadament_venda;

  -- Aquest test verifica l'error en fer una entrada a revisi per un empleat acomiadat.
  MEMBER PROCEDURE test_acomiadament_erevisio IS
    empleat_no_actiu EXCEPTION;
    PRAGMA EXCEPTION_INIT(empleat_no_actiu, -20101);
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    acomiadament('11102989E');
    entraRevisio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      100
    );
    m_util_object.ok(
      test_passed => FALSE,
      test_label  => 'Empleat acomiadat (entraRevisio)'
    );
  EXCEPTION
    WHEN empleat_no_actiu THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label  => 'Empleat acomiadat (entraRevisio)'
      );
  END test_acomiadament_erevisio;

  -- Aquest test verifica l'error en fer una sortida de revisi per un empleat acomiadat.
  MEMBER PROCEDURE test_acomiadament_srevisio IS
    empleat_no_actiu EXCEPTION;
    PRAGMA EXCEPTION_INIT(empleat_no_actiu, -20101);
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    entraRevisio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      100
    );
    acomiadament('11102989E');
    surtRevisio(
      '11102989E',
      '90909090D',
      '7888738395HG9898D',
      200
    );
    m_util_object.ok(
      test_passed => FALSE,
      test_label  => 'Empleat acomiadat (surtRevisio)'
    );
  EXCEPTION
    WHEN empleat_no_actiu THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label  => 'Empleat acomiadat (surtRevisio)'
      );
  END test_acomiadament_srevisio;

  -- Aquest test verifica l'error en fer una entrada de reparaci per un empleat acomiadat.
  MEMBER PROCEDURE test_acomiadament_ereparacio IS
    empleat_no_actiu EXCEPTION;
    PRAGMA EXCEPTION_INIT(empleat_no_actiu, -20101);
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    acomiadament('11102989E');
    entraReparacio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      100,
      'Sobrescalfaments'
    );
    m_util_object.ok(
      test_passed => FALSE,
      test_label  => 'Empleat acomiadat (entraReparacio)'
    );
  EXCEPTION
    WHEN empleat_no_actiu THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label  => 'Empleat acomiadat (entraReparacio)'
      );
  END test_acomiadament_ereparacio;

  -- Aquest test verifica l'error en fer una sortida de reparaci per un empleat acomiadat.
  MEMBER PROCEDURE test_acomiadament_sreparacio IS
    empleat_no_actiu EXCEPTION;
    PRAGMA EXCEPTION_INIT(empleat_no_actiu, -20101);
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    entraReparacio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      200,
      'Sobrescalfaments'
    );
    acomiadament('11102989E');
    surtReparacio(
      '11102989E',
      '90909090D',
      '7888738395HG9898D',
      200
    );
    m_util_object.ok(
      test_passed => FALSE,
      test_label  => 'Empleat acomiadat (surtReparacio)'
    );
  EXCEPTION
    WHEN empleat_no_actiu THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label  => 'Empleat acomiadat (surtReparacio)'
      );
  END test_acomiadament_sreparacio;

  -- Desprs de cada test, es desfan els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca101_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca101_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Es realitza un procediment per executar el test.
CREATE OR REPLACE PROCEDURE runTestsTasca101 IS
  ut_obj tasca101_test_obj := tasca101_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF