SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 104: Contractaci i acomiadaments (ETL).');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de l'ETL de la contractaci i l'acomiadament.
CREATE OR REPLACE TYPE tasca104_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE setup_testing,
  MEMBER PROCEDURE test_etl,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca104_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos de les validacions de l'ETL de la contractaci i l'acomiadament.
CREATE OR REPLACE TYPE body tasca104_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE setup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO empleats VALUES (
      '11102989E',
      'Marc',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102989E'
    );
    INSERT INTO mecanics VALUES (
      '11102989E'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
    INSERT INTO clients VALUES (
      '90909090D',
      'Manel',
      'c. Vigo',
      '986574833',
      '986574834',
      'manel@gmail.com'
    );
  END setup_testing;

  -- Aquest test prova l'ETL de la contractaci, tant per mecnics com per comercials, i de l'acomiadament.
  MEMBER PROCEDURE test_etl IS
    n_empleats   INT := 0;
    n_comercials INT := 0;
    n_mecanics   INT := 0;
    n_empleats2  INT := 0;
    codi_botiga  INT;
  BEGIN
    SELECT MAX(codi) INTO codi_botiga FROM botigues;
    contractacio('12102989E','Alex',codi_botiga,'M');
    SELECT COUNT(*) INTO n_empleats FROM concessionariDWH.dimEmpleats WHERE baixa IS NULL;
    concessionariDWH.etl;
    SELECT COUNT(*) INTO n_empleats2 FROM concessionariDWH.dimEmpleats WHERE baixa IS NULL;
    SELECT COUNT(*) INTO n_mecanics FROM concessionariDWH.dimMecanics;
    SELECT COUNT(*) INTO n_comercials FROM concessionariDWH.dimComercials;
    m_util_object.ok(
      test_passed => ( 
        n_empleats2 = n_empleats + 1 AND
        n_comercials = 0 AND
        n_mecanics = 1
      ),
      test_label  => 'Comptar una contractaci (mecnic)'
    );
    contractacio('13102989E','Carles',codi_botiga,'C');
    concessionariDWH.etl;
    SELECT COUNT(*) INTO n_empleats2 FROM concessionariDWH.dimEmpleats WHERE baixa IS NULL;
    SELECT COUNT(*) INTO n_mecanics FROM concessionariDWH.dimMecanics;
    SELECT COUNT(*) INTO n_comercials FROM concessionariDWH.dimComercials;
    m_util_object.ok(
      test_passed => ( 
        n_empleats2 = n_empleats + 2 AND
        n_comercials = 1 AND
        n_mecanics = 1
      ),
      test_label  => 'Comptar una contractaci (comercial)'
    );
    SELECT COUNT(*) INTO n_empleats FROM concessionariDWH.dimEmpleats WHERE baixa IS NULL;
    acomiadament('13102989E');
    concessionariDWH.etl;
    SELECT COUNT(*) INTO n_empleats2 FROM concessionariDWH.dimEmpleats WHERE baixa IS NULL;
    m_util_object.ok(
      test_passed => ( n_empleats2 = n_empleats - 1 ),
      test_label  => 'Comptar un acomiadament'
    );
  END test_etl;

  -- Desprs de cada test, es desfan els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    rollback;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca104_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca104_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Es realitza un procediment per executar el test.
CREATE OR REPLACE PROCEDURE runTeststasca104 IS
  ut_obj tasca104_test_obj := tasca104_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF