SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 107: Inauguraci i clausura de botigues.');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la inauguraci i clausura.
CREATE OR REPLACE TYPE tasca107_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE setup_testing,
  MEMBER PROCEDURE test_botiga,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca107_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos de les validacions de la inauguraci i clausura.
CREATE OR REPLACE TYPE BODY tasca107_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE setup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO empleats VALUES (
      '11102989E',
      'Marc',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102989E'
    );
    INSERT INTO mecanics VALUES (
      '11102989E'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
    INSERT INTO clients VALUES (
      '90909090D',
      'Manel',
      'c. Vigo',
      '986574833',
      '986574834',
      'manel@gmail.com'
    );
  END setup_testing;

  -- Aquest test comprova la creaci d'una nova botiga i la clausura d'una antiga, que substitueix la nova.
  MEMBER PROCEDURE test_botiga IS
    codi_botiga1 INT;
    codi_botiga2 INT;
    n_botigues   INT := 0;
    n_botigues2  INT := 0;
  BEGIN
    SELECT MAX(codi) INTO codi_botiga1 FROM botigues;
    inauguracio('Badal,82', 'C. Badal, 82', '937767769', '937767770', 'badal81@citroen.es');
    SELECT MAX(codi) INTO codi_botiga2 FROM botigues;
    m_util_object.ok(
      test_passed => ( codi_botiga2 = codi_botiga1 + 1 ),
      test_label  => 'Inaugurar una nova botiga'
    );
    SELECT COUNT(*) INTO n_botigues FROM botigues;
    clausura(codi_botiga1, codi_botiga2);
    SELECT COUNT(*) INTO n_botigues2 FROM botigues;
    m_util_object.ok(
      test_passed => ( n_botigues = n_botigues2 + 1 ),
      test_label  => 'Clausurar una botiga'
    );
  END test_botiga;

  -- Desprs de cada test, es desfan els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN 
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca107_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca107_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Es realitza un procediment per executar el test.
CREATE OR REPLACE PROCEDURE runTeststasca107 IS
  ut_obj tasca107_test_obj := tasca107_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF