SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 50: Compra.');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la compra.
CREATE OR REPLACE TYPE tasca50_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE startup_testing,
  MEMBER PROCEDURE test_compra,
  MEMBER PROCEDURE test_nouVenedor,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca50_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos de les validacions de la compra.
CREATE OR REPLACE TYPE BODY tasca50_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE startup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
  END startup_testing;

  -- Aquest test fa un parell de compres i comprova que s'han afegit.
  MEMBER PROCEDURE test_compra IS
    n_compres  INT := 0;
    n_compres2 INT := 0;
  BEGIN
    SELECT COUNT(*) INTO n_compres FROM compres;
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      NULL,
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    SELECT COUNT(*) INTO n_compres2 FROM compres;
    m_util_object.ok(
      test_passed => ( n_compres2 = n_compres + 1 ),
      test_label  => 'Afegir una compra (cotxe nou)'
    );
    SELECT COUNT(*) INTO n_compres FROM compres;
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9799D',
      '4234HHG',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanqu se',
      to_date('20121029','YYYYMMDD'),
      16000,
      120
    );
    SELECT COUNT(*) INTO n_compres2 FROM compres;
    m_util_object.ok(
      test_passed => ( n_compres2 = n_compres + 1 ),
      test_label  => 'Afegir una compra ms (cotxe segona m)'
    );
  END test_compra;

  -- Aquest test comprova que s'afegeix un nou venedor amb la funci nouVenedor.
  MEMBER PROCEDURE test_nouVenedor IS
    n_venedors  INT := 0;
    n_venedors2 INT := 0;
  BEGIN
    SELECT COUNT(*) INTO n_venedors FROM venedors;
    nouVenedor(
      '98686383Q',
      'Manuel',
      'Barcelona',
      '934567890',
      '934567891',
      'cotxes.usats@gmail.com'
    );
    SELECT COUNT(*) INTO n_venedors2 FROM venedors;
    m_util_object.ok(
      test_passed => ( n_venedors2 = n_venedors + 1 ),
      test_label  => 'Afegir un venedor'
    );
  END test_nouVenedor;

  -- Desprs de cada test, es desfan els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca50_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca50_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Es realitza un procediment per executar el test.
CREATE OR REPLACE PROCEDURE runTestsTasca50 IS
  ut_obj tasca50_test_obj := tasca50_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF