SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 60: Venda.');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la venda.
CREATE OR REPLACE TYPE tasca60_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE setup_testing,
  MEMBER PROCEDURE test_venda,
  MEMBER PROCEDURE test_nouClient,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca60_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos per les validacions de la venda
CREATE OR REPLACE TYPE BODY tasca60_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE setup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102989E'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
    INSERT INTO clients VALUES (
      '90909090D',
      'Manel',
      'c. Vigo',
      '986574833',
      '986574834',
      'manel@gmail.com'
    );
  END setup_testing;

  -- Aquest test fa una venda i comprova que s'hagi afegit.
  MEMBER PROCEDURE test_venda IS
    n_vendes  INT := 0;
    n_vendes2 INT := 0;
  BEGIN
    SELECT COUNT(*) INTO n_vendes FROM vendes;
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '1449HMC',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    SELECT COUNT(*) INTO n_vendes2 FROM vendes;
    m_util_object.ok(
      test_passed => ( n_vendes2 = n_vendes + 1 ),
      test_label  => 'Afegir una venda'
    );
  END test_venda;

  -- Aquest test crea un nou client i comprova que s'hagi afegit.
  MEMBER PROCEDURE test_nouClient IS
    n_clients  INT := 0;
    n_clients2 INT := 0;
  BEGIN
    SELECT COUNT(*) INTO n_clients FROM clients;
    nouClient(
      '98686383Q',
      'Manuel',
      'Barcelona',
      '934567890',
      '934567891',
      'manuel@gmail.com'
    );
    SELECT COUNT(*) INTO n_clients2 FROM clients;
    m_util_object.ok(
      test_passed => ( n_clients2 = n_clients + 1 ),
      test_label  => 'Afegir un client'
    );
  END test_nouClient;

  -- Per finalitzar cada test, es desfan tots els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca60_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca60_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per executar els tests.
CREATE OR REPLACE PROCEDURE runTestsTasca60 IS
  ut_obj tasca60_test_obj := tasca60_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF