SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 63: Compra i venda (ETL).');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la venda.
CREATE OR REPLACE TYPE tasca63_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE startup_testing,
  MEMBER PROCEDURE test_etl,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca63_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos per les validacions de l'ETL de la compra i la venda.
CREATE OR REPLACE TYPE BODY tasca63_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE startup_testing IS
  BEGIN
    INSERT INTO botigues   VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats   VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO empleats   VALUES (
      '10102990E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102990E'
    );
    INSERT INTO venedors   VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
  END startup_testing;

  -- Aquest test fa diferents vendes, executa l'ETL i comprova que es comptin correctament.
  MEMBER PROCEDURE test_etl IS
    n_compres  INT := 0;
    n_compres2 INT := 0;
    n_vendes   INT := 0;
    n_vendes2  INT := 0;
  BEGIN
    SELECT COUNT(*) INTO n_compres FROM concessionariDWH.fetsCompres;
    SELECT COUNT(*) INTO n_vendes  FROM concessionariDWH.fetsVendes;
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '1449HMB',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    nouClient(
      '09090909A',
      'Marc',
      'C. Suspiro Verde',
      '934567890',
      '934567890',
      'marc@gmail.com'
    );
    venda(
      '10102990E',
      '09090909A',
      '7888738395HG9798D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    concessionariDWH.etl;
    SELECT COUNT(*) INTO n_compres2 FROM concessionariDWH.fetsCompres;
    SELECT COUNT(*) INTO n_vendes2  FROM concessionariDWH.fetsVendes;
    m_util_object.ok(
      test_passed => ( n_compres2 = n_compres + 1 ),
      test_label  => 'Comptar una compra (cotxe nou)'
    );
    m_util_object.ok(
      test_passed => ( n_vendes2 = n_vendes + 1 ),
      test_label  => 'Comptar una venda'
    );
    n_compres := n_compres2;
    n_vendes  := n_vendes2;
    compra(
      '10102990E',
      '90909090D',
      '7888738395HG9799D',
      '1449HLK',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      to_date('20121029','YYYYMMDD'),
      16000,
      200
    );
    nouClient(
      '10090909A',
      'Marc',
      'C. Suspiro Verde',
      '934567890',
      '934567890',
      'marc@gmail.com'
    );
    venda(
      '10102990E',
      '10090909A',
      '7888738395HG9799D',
      to_date('20121029','YYYYMMDD'),
      16000
    );
    concessionariDWH.etl;
    SELECT COUNT(*) INTO n_compres2 FROM concessionariDWH.fetsCompres;
    SELECT COUNT(*) INTO n_vendes2  FROM concessionariDWH.fetsVendes;
    m_util_object.ok(
      test_passed => ( n_compres2 = n_compres + 1 ),
      test_label  => 'Comptar una segona compra (de segona m)'
    );
    m_util_object.ok(
      test_passed => ( n_vendes2 = n_vendes + 1 ),
      test_label  => 'Comptar una segona venda'
    );
  END test_etl;

  -- Per finalitzar cada test, es desfan tots els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca63_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca63_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per executar els tests.
CREATE OR REPLACE PROCEDURE runTestsTasca63 IS
  ut_obj tasca63_test_obj := tasca63_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF