SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 72: Revisions (ETL).');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la venda.
CREATE OR REPLACE TYPE tasca72_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE setup_testing,
  MEMBER PROCEDURE test_etl,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca72_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos per les validacions de l'ETL de la compra i la venda.
CREATE OR REPLACE TYPE BODY tasca72_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE setup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO empleats VALUES (
      '11102989E',
      'Marc',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102989E'
    );
    INSERT INTO mecanics VALUES (
      '11102989E'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
    INSERT INTO clients VALUES (
      '90909090D',
      'Manel',
      'c. Vigo',
      '986574833',
      '986574834',
      'manel@gmail.com'
    );
  END setup_testing;

  -- Es fa una entrada i es comprova que s'ha comptat desprs de l'ETL,
  -- desprs es fa una sortida i una entrada i tornar a fer l'ETL i comprovar.
  MEMBER PROCEDURE test_etl IS
    n_revisions       NUMBER  := 0;
    n_temps           NUMBER  := 0;
  BEGIN
    entraRevisio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      200
    );
    concessionariDWH.etl;
    SELECT COUNT(*) INTO n_revisions FROM concessionariDWH.fetsRevisions;
    SELECT SUM(tempsEmpleat) INTO n_temps FROM concessionariDWH.fetsRevisions;
    m_util_object.ok(
      test_passed => ( n_revisions = 1 AND n_temps = 0),
      test_label  => 'Comptar una revisi'
    );
    surtRevisio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      560
    );
    entraRevisio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      200
    );
    concessionariDWH.etl;
    SELECT COUNT(*) INTO n_revisions FROM concessionariDWH.fetsRevisions;
    SELECT SUM(tempsEmpleat) INTO n_temps FROM concessionariDWH.fetsRevisions;
    m_util_object.ok(
      test_passed => ( n_revisions = 2 AND n_temps > 0),
      test_label  => 'Comptar una segona revisi'
    );
  END test_etl;

  -- Per finalitzar cada test, es desfan tots els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca72_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca72_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per executar els tests.
CREATE OR REPLACE PROCEDURE runTestsTasca72 IS
  ut_obj tasca72_test_obj := tasca72_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF