SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 76: Reparacions.');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la venda.
CREATE OR REPLACE TYPE tasca76_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE setup_testing,
  MEMBER PROCEDURE test_reparacio_entrada,
  MEMBER PROCEDURE test_reparacio_sortida,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca76_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos per les validacions de la venda
CREATE OR REPLACE TYPE BODY tasca76_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE setup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO empleats VALUES (
      '11102989E',
      'Marc',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102989E'
    );
    INSERT INTO mecanics VALUES (
      '11102989E'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
    INSERT INTO clients VALUES (
      '90909090D',
      'Manel',
      'c. Vigo',
      '986574833',
      '986574834',
      'manel@gmail.com'
    );
  END setup_testing;

  -- Aquest test valida l'entrada a reparaci d'un vehicle ja existent i d'un que no ha vingut mai.
  MEMBER PROCEDURE test_reparacio_entrada IS
    n_kms          NUMBER  := 2900;
    n_kms2         NUMBER  := 0;
    n_vehicles     INT     := 0;
    n_vehicles2    INT     := 0;
    n_reparacions  INT     := 0;
    n_reparacions2 INT     := 0;
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    entraReparacio(
      '11102989E',
      '90909090D',
      '7888738395HG9798D',
      '4336HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      n_kms,
      'Sobrescalfaments'
    );
    SELECT kms INTO n_kms2 FROM vehicles WHERE vin='7888738395HG9798D';
    m_util_object.ok(
      test_passed => ( n_kms2 = n_kms ),
      test_label  => 'Entrar a reparaci un vehicle (existent)'
    );
    SELECT COUNT(vin) INTO n_vehicles FROM vehicles;
    SELECT COUNT(*) INTO n_reparacions FROM reparacions;
    entraReparacio(
      '11102989E',
      '90909090D',
      '7888738395HG9898D',
      '4336HFF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      n_kms,
      'Sobrescalfaments'
    );
    SELECT COUNT(*) INTO n_vehicles2 FROM vehicles;
    SELECT COUNT(*) INTO n_reparacions2 FROM reparacions;
    m_util_object.ok(
      test_passed => ( n_vehicles2 = n_vehicles + 1 AND
                       n_reparacions2 = n_reparacions + 1
                     ),
      test_label  => 'Entrar a reparaci un vehicle (no existent)'
    );
  END test_reparacio_entrada;

  -- Aquest test comprova la sortida d'un vehicle.
  MEMBER PROCEDURE test_reparacio_sortida IS
    n_reparacions  INT := 0;
    n_reparacions2 INT := 0;
    sortida      TIMESTAMP;
  BEGIN
    entraReparacio(
      '11102989E',
      '90909090D',
      '7888738395HG9898D',
      '4336HFF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      567,
      'Sobrescalfaments'
    );
    SELECT COUNT(*) INTO n_reparacions FROM reparacions;
    surtReparacio(
      '11102989E',
      '90909090D',
      '7888738395HG9898D',
      200
    );
    SELECT COUNT(*) INTO n_reparacions2 FROM reparacions WHERE import = 0;
    SELECT sortida INTO sortida FROM reparacions WHERE vinVehicle = '7888738395HG9898D';
    m_util_object.ok(
      test_passed => (
        n_reparacions2 = 0 AND
        n_reparacions = 1 AND
        sortida IS NOT NULL
      ),
      test_label  => 'Retirar un vehicle de reparaci'
    );
  END test_reparacio_sortida;

  -- Per finalitzar cada test, es desfan tots els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca76_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca76_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per executar els tests.
CREATE OR REPLACE PROCEDURE runTestsTasca76 IS
  ut_obj tasca76_test_obj := tasca76_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF