SET SERVEROUTPUT ON
SET TERMOUT ON
EXEC DBMS_OUTPUT.PUT_LINE('');
EXEC DBMS_OUTPUT.PUT_LINE('  Tests tasca 89: Matriculacions.');
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci de tipus per les validacions de la venda.
CREATE OR REPLACE TYPE tasca89_test_obj UNDER sys.pluto_obj(
  MEMBER PROCEDURE setup_testing,
  MEMBER PROCEDURE test_matricula_manual,
  MEMBER PROCEDURE test_matricula_compra,
  MEMBER PROCEDURE test_matricula_venda,
  MEMBER PROCEDURE test_matricula_venda2,
  MEMBER PROCEDURE teardown_testing,
  CONSTRUCTOR FUNCTION tasca89_test_obj
    RETURN SELF AS RESULT
) INSTANTIABLE NOT FINAL;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Declaraci del cos per les validacions de la venda
CREATE OR REPLACE TYPE BODY tasca89_test_obj IS
  -- En el setup, es creen alguns registres que els tests poquin utilitzar.
  MEMBER PROCEDURE setup_testing IS
  BEGIN
    INSERT INTO botigues VALUES (
      NULL,
      'Badal,81',
      'C. Badal, 81',
      '937767769',
      '937767770',
      'badal81@citroen.es'
    );
    INSERT INTO empleats VALUES (
      '10102989E',
      'Josep',
      botigues_codi.CURRVAL,
      'Y'
    );
    INSERT INTO comercials VALUES (
      '10102989E'
    );
    INSERT INTO venedors VALUES (
      '90909090D',
      'Citroen España',
      'c. Vigo',
      '986574833',
      '986574834',
      'fabrica@citroen.es'
    );
    INSERT INTO clients VALUES (
      '90909090D',
      'Manel',
      'c. Vigo',
      '986574833',
      '986574834',
      'manel@gmail.com'
    );
  END setup_testing;

  -- Aquest test verifica l'addici d'una matrcula manualment.
  MEMBER PROCEDURE test_matricula_manual IS
    n_matricules          INT := 0;
    n_matricules2         INT := 0;
    n_matricules_lliures  INT := 0;
    n_matricules_lliures2 INT := 0;
  BEGIN
    SELECT COUNT(*) INTO n_matricules         FROM matricules;
    SELECT COUNT(*) INTO n_matricules_lliures FROM matricules WHERE vinVehicle IS NULL;
    novaMatricula('4234HHG');
    SELECT COUNT(*) INTO n_matricules2          FROM matricules;
    SELECT COUNT(*) INTO n_matricules_lliures2  FROM matricules WHERE vinVehicle IS NULL;
    m_util_object.ok(
      test_passed => (
        n_matricules2         = n_matricules + 1
        AND
        n_matricules_lliures2 = n_matricules_lliures + 1
      ),
      test_label  => 'Afegir una matrcula (manualment)'
    );
    novaMatricula('4234HHG');
    m_util_object.ok(
      test_passed => FALSE,
      test_label => 'Matrcula duplicada'
    );
  EXCEPTION
    WHEN dup_val_on_index THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label => 'Matrcula duplicada'
      );
  END test_matricula_manual;

  -- Aquest test verifica que en comprar un vehicle matriculat, la matrcula s'afegeix, evitant duplicats.
  MEMBER PROCEDURE test_matricula_compra IS
    n_matricules          INT := 0;
    n_matricules2         INT := 0;
    n_matricules_lliures  INT := 0;
    n_matricules_lliures2 INT := 0;
  BEGIN
    SELECT COUNT(*) INTO n_matricules FROM matricules;
    SELECT COUNT(*) INTO n_matricules_lliures FROM matricules WHERE vinVehicle IS NULL;
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      '4563HLF',
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    SELECT COUNT(*) INTO n_matricules2 FROM matricules;
    SELECT COUNT(*) INTO n_matricules_lliures2 FROM matricules WHERE vinVehicle IS NULL;
    m_util_object.ok(
      test_passed => (
        n_matricules2 = n_matricules + 1 
        AND
        n_matricules_lliures2 = n_matricules_lliures
      ),
      test_label  => 'Afegir una matricula (compra cotxe nou)'
    );
    novaMatricula('4563HLF');
    m_util_object.ok(
      test_passed => FALSE,
      test_label => 'Matrcula duplicada'
    );
  EXCEPTION
    WHEN dup_val_on_index THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label => 'Matrcula duplicada'
      );
  END test_matricula_compra;

  -- Aquest test verifica que les vendes interaccionen b amb les matrcules.
  MEMBER PROCEDURE test_matricula_venda IS
    n_matricules_lliures  INT := 0;
    n_matricules_lliures2 INT := 0;
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      NULL,
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9799D',
      NULL,
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    novaMatricula('4563HLF');
    SELECT COUNT(*) INTO n_matricules_lliures FROM matricules WHERE vinVehicle IS NULL;
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9798D',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    SELECT COUNT(*) INTO n_matricules_lliures2 FROM matricules WHERE vinVehicle IS NULL;
    m_util_object.ok(
      test_passed => ( n_matricules_lliures2 = n_matricules_lliures - 1 ),
      test_label => 'Assignaci automtica de matrcula en venda'
    );
  END test_matricula_venda;

  -- Aquest test verifica que les vendes interaccionen b amb les matrcules.
  MEMBER PROCEDURE test_matricula_venda2 IS
  BEGIN
    compra(
      '10102989E',
      '90909090D',
      '7888738395HG9799D',
      NULL,
      'Citroën',
      'C4',
      '1.9 HDI',
      'A/C',
      'Blanco Blanquisse',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    venda(
      '10102989E',
      '90909090D',
      '7888738395HG9799D',
      TO_DATE('20121029','YYYYMMDD'),
      16000
    );
    m_util_object.ok(
      test_passed => FALSE,
      test_label => 'Manquen matrcules'
    );
  EXCEPTION
    WHEN no_data_found THEN
      m_util_object.ok(
        test_passed => TRUE,
        test_label => 'Manquen matrcules'
      );
  END test_matricula_venda2;

  -- Per finalitzar cada test, es desfan tots els canvis.
  MEMBER PROCEDURE teardown_testing IS
  BEGIN
    ROLLBACK;
  END teardown_testing;

  -- Aquest s el constructor.
  CONSTRUCTOR FUNCTION tasca89_test_obj
    RETURN SELF AS RESULT IS
  BEGIN
    m_util_object := sys.pluto_util_obj();
    RETURN;
  END tasca89_test_obj;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF

-- Crear el procediment per executar els tests.
CREATE OR REPLACE PROCEDURE runTestsTasca89 IS
  ut_obj tasca89_test_obj := tasca89_test_obj();
BEGIN
  ut_obj.run_tests;
END;
/

SET SERVEROUTPUT ON
SET TERMOUT ON
SHOW ERRORS;
SET TERMOUT OFF
SET SERVEROUTPUT OFF